@@ -39,7 +39,6 @@ def read_language_file(filename, strings_found, errors):
     skip = SkipType.NONE
     length = 0
     common_prefix = ""
-    last_tiny_string = ""
 
     with open(filename) as fp:
         for line in fp.readlines():
@@ -114,17 +113,6 @@ def read_language_file(filename, strings_found, errors):
             name = line.split(":")[0].strip()
             strings_defined.append(name)
 
-            # If a string ends on _TINY or _SMALL, it can be the {TINY} variant.
-            # Check for this by some fuzzy matching.
-            if name.endswith(("_SMALL", "_TINY")):
-                last_tiny_string = name
-            elif last_tiny_string:
-                matching_name = "_".join(last_tiny_string.split("_")[:-1])
-                if name == matching_name:
-                    strings_found.add(last_tiny_string)
-            else:
-                last_tiny_string = ""
-
             if skip == SkipType.EXTERNAL:
                 strings_found.add(name)
                 skip = SkipType.LENGTH
@@ -170,7 +158,7 @@ def scan_source_files(path, strings_found):
         # Most files we can just open, but some use magic, that requires the
         # G++ preprocessor before we can make sense out of it.
         if new_path == "src/table/cargo_const.h":
-            p = subprocess.run(["g++", "-E", new_path], stdout=subprocess.PIPE)
+            p = subprocess.run(["g++", "-E", "-DCHECK_UNUSED_STRINGS", new_path], stdout=subprocess.PIPE)
             output = p.stdout.decode()
         else:
             with open(new_path) as fp:
