@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file base_station_base.h Base classes/functions for base stations. */
@@ -16,31 +16,25 @@
 #include "station_map.h"
 #include "timer/timer_game_calendar.h"
 
-typedef Pool<BaseStation, StationID, 32, 64000> StationPool;
+typedef Pool<BaseStation, StationID, 32> StationPool;
 extern StationPool _station_pool;
 
-struct StationSpecList {
-	const StationSpec *spec;
-	uint32_t grfid;      ///< GRF ID of this custom station
-	uint16_t localidx; ///< Station ID within GRF of station
-};
-
-struct RoadStopSpecList {
-	const RoadStopSpec *spec;
-	uint32_t grfid;      ///< GRF ID of this custom road stop
-	uint16_t localidx; ///< Station ID within GRF of road stop
+template <typename T>
+struct SpecMapping {
+	const T *spec = nullptr; ///< Custom spec.
+	uint32_t grfid = 0; ///< GRF ID of this custom spec.
+	uint16_t localidx = 0; ///< Local ID within GRF of this custom spec.
 };
 
 struct RoadStopTileData {
-	TileIndex tile;
-	uint8_t random_bits;
-	uint8_t animation_frame;
+	TileIndex tile = INVALID_TILE;
+	uint8_t random_bits = 0;
+	uint8_t animation_frame = 0;
 };
 
 /** StationRect - used to track station spread out rectangle - cheaper than scanning whole map */
 struct StationRect : public Rect {
-	enum StationRectMode
-	{
+	enum StationRectMode : uint8_t {
 		ADD_TEST = 0,
 		ADD_TRY,
 		ADD_FORCE
@@ -62,44 +56,40 @@ struct StationRect : public Rect {
 
 /** Base class for all station-ish types */
 struct BaseStation : StationPool::PoolItem<&_station_pool> {
-	TileIndex xy;                   ///< Base tile of the station
-	TrackedViewportSign sign;       ///< NOSAVE: Dimensions of sign
-	byte delete_ctr;                ///< Delete counter. If greater than 0 then it is decremented until it reaches 0; the waypoint is then is deleted.
+	TileIndex xy = INVALID_TILE; ///< Base tile of the station
+	TrackedViewportSign sign{}; ///< NOSAVE: Dimensions of sign
+	uint8_t delete_ctr = 0; ///< Delete counter. If greater than 0 then it is decremented until it reaches 0; the waypoint is then is deleted.
 
-	std::string name;               ///< Custom name
-	StringID string_id;             ///< Default name (town area) of station
+	std::string name{}; ///< Custom name
+	StringID string_id = INVALID_STRING_ID; ///< Default name (town area) of station
 	mutable std::string cached_name; ///< NOSAVE: Cache of the resolved name of the station, if not using a custom name
 
-	Town *town;                     ///< The town this station is associated with
-	Owner owner;                    ///< The owner of this station
-	StationFacility facilities;     ///< The facilities that this station has
+	Town *town = nullptr; ///< The town this station is associated with
+	Owner owner = INVALID_OWNER; ///< The owner of this station
+	StationFacilities facilities{}; ///< The facilities that this station has
 
-	std::vector<StationSpecList> speclist;           ///< List of rail station specs of this station.
-	std::vector<RoadStopSpecList> roadstop_speclist; ///< List of road stop specs of this station
+	std::vector<SpecMapping<StationSpec>> speclist{}; ///< List of rail station specs of this station.
+	std::vector<SpecMapping<RoadStopSpec>> roadstop_speclist{}; ///< List of road stop specs of this station
 
-	TimerGameCalendar::Date build_date; ///< Date of construction
+	TimerGameCalendar::Date build_date{}; ///< Date of construction
 
-	uint16_t random_bits;             ///< Random bits assigned to this station
-	byte waiting_triggers;          ///< Waiting triggers (NewGRF) for this station
-	uint8_t cached_anim_triggers;                ///< NOSAVE: Combined animation trigger bitmask, used to determine if trigger processing should happen.
-	uint8_t cached_roadstop_anim_triggers;       ///< NOSAVE: Combined animation trigger bitmask for road stops, used to determine if trigger processing should happen.
-	CargoTypes cached_cargo_triggers;          ///< NOSAVE: Combined cargo trigger bitmask
-	CargoTypes cached_roadstop_cargo_triggers; ///< NOSAVE: Combined cargo trigger bitmask for road stops
+	uint16_t random_bits = 0; ///< Random bits assigned to this station
+	StationRandomTriggers waiting_random_triggers; ///< Waiting triggers (NewGRF), shared by all station parts/tiles, road stops, ... essentially useless and broken by design.
+	StationAnimationTriggers cached_anim_triggers; ///< NOSAVE: Combined animation trigger bitmask, used to determine if trigger processing should happen.
+	StationAnimationTriggers cached_roadstop_anim_triggers; ///< NOSAVE: Combined animation trigger bitmask for road stops, used to determine if trigger processing should happen.
+	CargoTypes cached_cargo_triggers{}; ///< NOSAVE: Combined cargo trigger bitmask
+	CargoTypes cached_roadstop_cargo_triggers{}; ///< NOSAVE: Combined cargo trigger bitmask for road stops
 
-	TileArea train_station;         ///< Tile area the train 'station' part covers
-	StationRect rect;               ///< NOSAVE: Station spread out rectangle maintained by StationRect::xxx() functions
+	TileArea train_station{INVALID_TILE, 0, 0}; ///< Tile area the train 'station' part covers
+	StationRect rect{}; ///< NOSAVE: Station spread out rectangle maintained by StationRect::xxx() functions
 
-	std::vector<RoadStopTileData> custom_roadstop_tile_data; ///< List of custom road stop tile data
+	std::vector<RoadStopTileData> custom_roadstop_tile_data{}; ///< List of custom road stop tile data
 
 	/**
 	 * Initialize the base station.
 	 * @param tile The location of the station sign
 	 */
-	BaseStation(TileIndex tile) :
-		xy(tile),
-		train_station(INVALID_TILE, 0, 0)
-	{
-	}
+	BaseStation(TileIndex tile) : xy(tile) {}
 
 	virtual ~BaseStation();
 
@@ -118,7 +108,7 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 	 * @param available will return false if ever the variable asked for does not exist
 	 * @return the value stored in the corresponding variable
 	 */
-	virtual uint32_t GetNewGRFVariable(const struct ResolverObject &object, byte variable, byte parameter, bool *available) const = 0;
+	virtual uint32_t GetNewGRFVariable(const struct ResolverObject &object, uint8_t variable, uint8_t parameter, bool &available) const = 0;
 
 	/**
 	 * Update the coordinated of the sign (as shown in the viewport).
@@ -140,11 +130,10 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 
 	/**
 	 * Get the tile area for a given station type.
-	 * @param ta tile area to fill.
 	 * @param type the type of the area
+	 * @return The tile area.
 	 */
-	virtual void GetTileArea(TileArea *ta, StationType type) const = 0;
-
+	virtual TileArea GetTileArea(StationType type) const = 0;
 
 	/**
 	 * Obtain the length of a platform
@@ -181,10 +170,10 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 	 */
 	inline bool IsInUse() const
 	{
-		return (this->facilities & ~FACIL_WAYPOINT) != 0;
+		return this->facilities.Any({StationFacility::Train, StationFacility::TruckStop, StationFacility::BusStop, StationFacility::Airport, StationFacility::Dock});
 	}
 
-	inline byte GetRoadStopRandomBits(TileIndex tile) const
+	inline uint8_t GetRoadStopRandomBits(TileIndex tile) const
 	{
 		for (const RoadStopTileData &tile_data : this->custom_roadstop_tile_data) {
 			if (tile_data.tile == tile) return tile_data.random_bits;
@@ -192,7 +181,7 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 		return 0;
 	}
 
-	inline byte GetRoadStopAnimationFrame(TileIndex tile) const
+	inline uint8_t GetRoadStopAnimationFrame(TileIndex tile) const
 	{
 		for (const RoadStopTileData &tile_data : this->custom_roadstop_tile_data) {
 			if (tile_data.tile == tile) return tile_data.animation_frame;
@@ -201,11 +190,11 @@ struct BaseStation : StationPool::PoolItem<&_station_pool> {
 	}
 
 private:
-	void SetRoadStopTileData(TileIndex tile, byte data, bool animation);
+	bool SetRoadStopTileData(TileIndex tile, uint8_t data, bool animation);
 
 public:
-	inline void SetRoadStopRandomBits(TileIndex tile, byte random_bits) { this->SetRoadStopTileData(tile, random_bits, false); }
-	inline void SetRoadStopAnimationFrame(TileIndex tile, byte frame) { this->SetRoadStopTileData(tile, frame, true); }
+	inline void SetRoadStopRandomBits(TileIndex tile, uint8_t random_bits) { this->SetRoadStopTileData(tile, random_bits, false); }
+	inline bool SetRoadStopAnimationFrame(TileIndex tile, uint8_t frame) { return this->SetRoadStopTileData(tile, frame, true); }
 	void RemoveRoadStopTileData(TileIndex tile);
 
 	static void PostDestructor(size_t index);
@@ -220,13 +209,13 @@ private:
  */
 template <class T, bool Tis_waypoint>
 struct SpecializedStation : public BaseStation {
-	static const StationFacility EXPECTED_FACIL = Tis_waypoint ? FACIL_WAYPOINT : FACIL_NONE; ///< Specialized type
+	static constexpr StationFacilities EXPECTED_FACIL = Tis_waypoint ? StationFacility::Waypoint : StationFacilities{}; ///< Specialized type
 
 	/**
 	 * Set station type correctly
 	 * @param tile The base tile of the station.
 	 */
-	inline SpecializedStation<T, Tis_waypoint>(TileIndex tile) :
+	inline SpecializedStation(TileIndex tile) :
 			BaseStation(tile)
 	{
 		this->facilities = EXPECTED_FACIL;
@@ -239,7 +228,7 @@ struct SpecializedStation : public BaseStation {
 	 */
 	static inline bool IsExpected(const BaseStation *st)
 	{
-		return (st->facilities & FACIL_WAYPOINT) == EXPECTED_FACIL;
+		return st->facilities.Test(StationFacility::Waypoint) == Tis_waypoint;
 	}
 
 	/**
@@ -247,16 +236,16 @@ struct SpecializedStation : public BaseStation {
 	 * @param index tested index
 	 * @return is this index valid index of T?
 	 */
-	static inline bool IsValidID(size_t index)
+	static inline bool IsValidID(auto index)
 	{
 		return BaseStation::IsValidID(index) && IsExpected(BaseStation::Get(index));
 	}
 
 	/**
 	 * Gets station with given index
-	 * @return pointer to station with given index casted to T *
+	 * @return pointer to station with given index cast to T *
 	 */
-	static inline T *Get(size_t index)
+	static inline T *Get(auto index)
 	{
 		return (T *)BaseStation::Get(index);
 	}
@@ -265,7 +254,7 @@ struct SpecializedStation : public BaseStation {
 	 * Returns station if the index is a valid index for this station type
 	 * @return pointer to station with given index if it's a station of this type
 	 */
-	static inline T *GetIfValid(size_t index)
+	static inline T *GetIfValid(auto index)
 	{
 		return IsValidID(index) ? Get(index) : nullptr;
 	}
@@ -310,4 +299,14 @@ struct SpecializedStation : public BaseStation {
 	static Pool::IterateWrapper<T> Iterate(size_t from = 0) { return Pool::IterateWrapper<T>(from); }
 };
 
+/**
+ * Get spec mapping list for each supported custom spec type.
+ * @tparam T Spec type.
+ * @param bst Station of custom spec list.
+ * @return Speclist of custom spec type.
+ */
+template <class T> std::vector<SpecMapping<T>> &GetStationSpecList(BaseStation *bst);
+template <> inline std::vector<SpecMapping<StationSpec>> &GetStationSpecList<StationSpec>(BaseStation *bst) { return bst->speclist; }
+template <> inline std::vector<SpecMapping<RoadStopSpec>> &GetStationSpecList<RoadStopSpec>(BaseStation *bst) { return bst->roadstop_speclist; }
+
 #endif /* BASE_STATION_BASE_H */
