@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file clear_map.h Map accessors for 'clear' tiles */
@@ -16,12 +16,12 @@
 /**
  * Ground types. Valid densities in comments after the enum.
  */
-enum ClearGround {
+enum ClearGround : uint8_t {
 	CLEAR_GRASS  = 0, ///< 0-3
 	CLEAR_ROUGH  = 1, ///< 3
 	CLEAR_ROCKS  = 2, ///< 3
 	CLEAR_FIELDS = 3, ///< 3
-	CLEAR_SNOW   = 4, ///< 0-3
+	CLEAR_SNOW   = 4, ///< 0-3 (Not stored in map.)
 	CLEAR_DESERT = 5, ///< 1,3
 };
 
@@ -38,18 +38,6 @@ inline bool IsSnowTile(Tile t)
 	return HasBit(t.m3(), 4);
 }
 
-/**
- * Get the type of clear tile but never return CLEAR_SNOW.
- * @param t the tile to get the clear ground type of
- * @pre IsTileType(t, MP_CLEAR)
- * @return the ground type
- */
-inline ClearGround GetRawClearGround(Tile t)
-{
-	assert(IsTileType(t, MP_CLEAR));
-	return (ClearGround)GB(t.m5(), 2, 3);
-}
-
 /**
  * Get the type of clear tile.
  * @param t the tile to get the clear ground type of
@@ -58,8 +46,8 @@ inline ClearGround GetRawClearGround(Tile t)
  */
 inline ClearGround GetClearGround(Tile t)
 {
-	if (IsSnowTile(t)) return CLEAR_SNOW;
-	return GetRawClearGround(t);
+	assert(IsTileType(t, MP_CLEAR));
+	return static_cast<ClearGround>(GB(t.m5(), 2, 3));
 }
 
 /**
@@ -207,7 +195,7 @@ inline IndustryID GetIndustryIndexOfField(Tile t)
 inline void SetIndustryIndexOfField(Tile t, IndustryID i)
 {
 	assert(GetClearGround(t) == CLEAR_FIELDS);
-	t.m2() = i;
+	t.m2() = i.base();
 }
 
 
@@ -282,11 +270,11 @@ inline void MakeField(Tile t, uint field_type, IndustryID industry)
 	SetTileType(t, MP_CLEAR);
 	t.m1() = 0;
 	SetTileOwner(t, OWNER_NONE);
-	t.m2() = industry;
+	t.m2() = industry.base();
 	t.m3() = field_type;
 	t.m4() = 0 << 5 | 0 << 2;
 	SetClearGroundDensity(t, CLEAR_FIELDS, 3);
-	SB(t.m6(), 2, 4, 0);
+	SB(t.m6(), 2, 6, 0);
 	t.m7() = 0;
 	t.m8() = 0;
 }
@@ -295,13 +283,13 @@ inline void MakeField(Tile t, uint field_type, IndustryID industry)
  * Make a snow tile.
  * @param t the tile to make snowy
  * @param density The density of snowiness.
- * @pre GetClearGround(t) != CLEAR_SNOW
+ * @pre !IsSnowTile(t)
  */
 inline void MakeSnow(Tile t, uint density = 0)
 {
-	assert(GetClearGround(t) != CLEAR_SNOW);
+	assert(!IsSnowTile(t));
 	SetBit(t.m3(), 4);
-	if (GetRawClearGround(t) == CLEAR_FIELDS) {
+	if (GetClearGround(t) == CLEAR_FIELDS) {
 		SetClearGroundDensity(t, CLEAR_GRASS, density);
 	} else {
 		SetClearDensity(t, density);
@@ -311,11 +299,11 @@ inline void MakeSnow(Tile t, uint density = 0)
 /**
  * Clear the snow from a tile and return it to its previous type.
  * @param t the tile to clear of snow
- * @pre GetClearGround(t) == CLEAR_SNOW
+ * @pre IsSnowTile(t)
  */
 inline void ClearSnow(Tile t)
 {
-	assert(GetClearGround(t) == CLEAR_SNOW);
+	assert(IsSnowTile(t));
 	ClrBit(t.m3(), 4);
 	SetClearDensity(t, 3);
 }
