@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file gfx_layout.h Functions related to laying out the texts. */
@@ -10,11 +10,11 @@
 #ifndef GFX_LAYOUT_H
 #define GFX_LAYOUT_H
 
+#include "misc/lrucache.hpp"
 #include "fontcache.h"
 #include "gfx_func.h"
 #include "core/math_func.hpp"
 
-#include <stack>
 #include <string_view>
 
 /**
@@ -24,19 +24,20 @@
 struct FontState {
 	FontSize fontsize;       ///< Current font size.
 	TextColour cur_colour;   ///< Current text colour.
-
-	std::stack<TextColour, std::vector<TextColour>> colour_stack; ///< Stack of colours to assist with colour switching.
+	std::vector<TextColour> colour_stack; ///< Stack of colours to assist with colour switching.
 
 	FontState() : fontsize(FS_END), cur_colour(TC_INVALID) {}
 	FontState(TextColour colour, FontSize fontsize) : fontsize(fontsize), cur_colour(colour) {}
 
+	auto operator<=>(const FontState &) const = default;
+
 	/**
 	 * Switch to new colour \a c.
 	 * @param c New colour to use.
 	 */
 	inline void SetColour(TextColour c)
 	{
-		assert((c & TC_COLOUR_MASK) >= TC_BLUE && (c & TC_COLOUR_MASK) <= TC_BLACK);
+		assert(((c & TC_COLOUR_MASK) >= TC_BLUE && (c & TC_COLOUR_MASK) <= TC_BLACK) || (c & TC_COLOUR_MASK) == TC_INVALID);
 		assert((c & (TC_COLOUR_MASK | TC_FLAGS_MASK)) == c);
 		if ((this->cur_colour & TC_FORCED) == 0) this->cur_colour = c;
 	}
@@ -47,8 +48,8 @@ struct FontState {
 	inline void PopColour()
 	{
 		if (colour_stack.empty()) return;
-		SetColour(colour_stack.top());
-		colour_stack.pop();
+		SetColour(colour_stack.back());
+		colour_stack.pop_back();
 	}
 
 	/**
@@ -56,7 +57,7 @@ struct FontState {
 	 */
 	inline void PushColour()
 	{
-		colour_stack.push(this->cur_colour);
+		colour_stack.push_back(this->cur_colour);
 	}
 
 	/**
@@ -69,6 +70,27 @@ struct FontState {
 	}
 };
 
+template <typename T> struct std::hash<std::vector<T>> {
+	size_t operator()(const std::vector<T> &vec) const
+	{
+		/* This is not an optimal hash algorithm but in most cases this is empty and therefore the same anyway. */
+		return std::transform_reduce(std::begin(vec), std::end(vec),
+			std::hash<size_t>{}(std::size(vec)),
+			[](const size_t &a, const size_t &b) -> size_t { return a ^ b; },
+			[](const T &x) -> size_t { return std::hash<T>{}(x); });
+	}
+};
+
+template <> struct std::hash<FontState> {
+	std::size_t operator()(const FontState &state) const noexcept
+	{
+		size_t h1 = std::hash<FontSize>{}(state.fontsize);
+		size_t h2 = std::hash<TextColour>{}(state.cur_colour);
+		size_t h3 = std::hash<std::vector<TextColour>>{}(state.colour_stack);
+		return h1 ^ (h2 << 1) ^ (h3 << 2);
+	}
+};
+
 /**
  * Container with information about a font.
  */
@@ -81,7 +103,7 @@ public:
 };
 
 /** Mapping from index to font. The pointer is owned by FontColourMap. */
-using FontMap = std::map<int, Font *>;
+using FontMap = std::vector<std::pair<int, Font *>>;
 
 /**
  * Interface to glue fallback and normal layouter into one.
@@ -90,16 +112,29 @@ class ParagraphLayouter {
 public:
 	virtual ~ParagraphLayouter() = default;
 
+	/** Position of a glyph within a VisualRun. */
+	class Position {
+	public:
+		int16_t left; ///< Left-most position of glyph.
+		int16_t right; ///< Right-most position of glyph.
+		int16_t top; ///< Top-most position of glyph.
+
+		constexpr inline Position(int16_t left, int16_t right, int16_t top) : left(left), right(right), top(top) { }
+
+		/** Conversion from a single point to a Position. */
+		constexpr inline Position(const Point &pt) : left(pt.x), right(pt.x), top(pt.y) { }
+	};
+
 	/** Visual run contains data about the bit of text with the same font. */
 	class VisualRun {
 	public:
 		virtual ~VisualRun() = default;
 		virtual const Font *GetFont() const = 0;
 		virtual int GetGlyphCount() const = 0;
-		virtual const std::vector<GlyphID> &GetGlyphs() const = 0;
-		virtual const std::vector<Point> &GetPositions() const = 0;
+		virtual std::span<const GlyphID> GetGlyphs() const = 0;
+		virtual std::span<const Position> GetPositions() const = 0;
 		virtual int GetLeading() const = 0;
-		virtual const std::vector<int> &GetGlyphToCharMap() const = 0;
+		virtual std::span<const int> GetGlyphToCharMap() const = 0;
 	};
 
 	/** A single line worth of VisualRuns. */
@@ -122,7 +157,7 @@ public:
  *
  * It also accounts for the memory allocations and frees.
  */
-class Layouter : public std::vector<std::unique_ptr<const ParagraphLayouter::Line>> {
+class Layouter : public std::vector<const ParagraphLayouter::Line *> {
 	std::string_view string; ///< Pointer to the original string.
 
 	/** Key into the linecache */
@@ -132,40 +167,41 @@ class Layouter : public std::vector<std::unique_ptr<const ParagraphLayouter::Lin
 	};
 
 	struct LineCacheQuery {
-		FontState state_before;  ///< Font state at the beginning of the line.
+		const FontState &state_before; ///< Font state at the beginning of the line.
 		std::string_view str;    ///< Source string of the line (including colour and font size codes).
 	};
 
-	/** Comparator for std::map */
-	struct LineCacheCompare {
-		using is_transparent = void; ///< Enable map queries with various key types
+	friend struct std::hash<Layouter::LineCacheQuery>;
+	struct LineCacheHash;
 
-		/** Comparison operator for LineCacheKey and LineCacheQuery */
-		template<typename Key1, typename Key2>
-		bool operator()(const Key1 &lhs, const Key2 &rhs) const
+	struct LineCacheEqualTo {
+		using is_transparent = void;
+
+		template <typename Tlhs, typename Trhs>
+		bool operator()(const Tlhs &lhs, const Trhs &rhs) const
 		{
-			if (lhs.state_before.fontsize != rhs.state_before.fontsize) return lhs.state_before.fontsize < rhs.state_before.fontsize;
-			if (lhs.state_before.cur_colour != rhs.state_before.cur_colour) return lhs.state_before.cur_colour < rhs.state_before.cur_colour;
-			if (lhs.state_before.colour_stack != rhs.state_before.colour_stack) return lhs.state_before.colour_stack < rhs.state_before.colour_stack;
-			return lhs.str < rhs.str;
+			return lhs.state_before == rhs.state_before && lhs.str == rhs.str;
 		}
 	};
+
 public:
 	/** Item in the linecache */
 	struct LineCacheItem {
+		/* Due to the type of data in the buffer differing depending on the Layouter, we need to pass our own deleter routine. */
+		using Buffer = std::unique_ptr<void, void(*)(void *)>;
 		/* Stuff that cannot be freed until the ParagraphLayout is freed */
-		void *buffer;              ///< Accessed by our ParagraphLayout::nextLine.
+		Buffer buffer{nullptr, [](void *){}}; ///< Accessed by our ParagraphLayout::nextLine.
 		FontMap runs;              ///< Accessed by our ParagraphLayout::nextLine.
 
 		FontState state_after;     ///< Font state after the line.
-		ParagraphLayouter *layout; ///< Layout of the line.
+		std::unique_ptr<ParagraphLayouter> layout = nullptr; ///< Layout of the line.
 
-		LineCacheItem() : buffer(nullptr), layout(nullptr) {}
-		~LineCacheItem() { delete layout; free(buffer); }
+		std::vector<std::unique_ptr<const ParagraphLayouter::Line>> cached_layout{}; ///< Cached results of line layouting.
+		int cached_width = 0; ///< Width used for the cached layout.
 	};
 private:
-	typedef std::map<LineCacheKey, LineCacheItem, LineCacheCompare> LineCache;
-	static LineCache *linecache;
+	using LineCache = LRUCache<LineCacheKey, LineCacheItem, LineCacheHash, LineCacheEqualTo>;
+	static std::unique_ptr<LineCache> linecache;
 
 	static LineCacheItem &GetCachedParagraphLayout(std::string_view str, const FontState &state);
 
@@ -174,15 +210,33 @@ private:
 public:
 	static Font *GetFont(FontSize size, TextColour colour);
 
-	Layouter(std::string_view str, int maxw = INT32_MAX, TextColour colour = TC_FROMSTRING, FontSize fontsize = FS_NORMAL);
+	Layouter(std::string_view str, int maxw = INT32_MAX, FontSize fontsize = FS_NORMAL);
 	Dimension GetBounds();
-	Point GetCharPosition(std::string_view::const_iterator ch) const;
+	ParagraphLayouter::Position GetCharPosition(std::string_view::const_iterator ch) const;
 	ptrdiff_t GetCharAtPosition(int x, size_t line_index) const;
 
 	static void Initialize();
 	static void ResetFontCache(FontSize size);
 	static void ResetLineCache();
-	static void ReduceLineCache();
+};
+
+ParagraphLayouter::Position GetCharPosInString(std::string_view str, size_t pos, FontSize start_fontsize = FS_NORMAL);
+ptrdiff_t GetCharAtPosition(std::string_view str, int x, FontSize start_fontsize = FS_NORMAL);
+
+template <> struct std::hash<Layouter::LineCacheQuery> {
+	std::size_t operator()(const Layouter::LineCacheQuery &state) const noexcept
+	{
+		size_t h1 = std::hash<std::string_view>{}(state.str);
+		size_t h2 = std::hash<FontState>{}(state.state_before);
+		return h1 ^ (h2 << 1);
+	}
+};
+
+struct Layouter::LineCacheHash {
+	using is_transparent = void;
+
+	std::size_t operator()(const Layouter::LineCacheKey &query) const { return std::hash<Layouter::LineCacheQuery>{}(LineCacheQuery{query.state_before, query.str}); }
+	std::size_t operator()(const Layouter::LineCacheQuery &query) const { return std::hash<Layouter::LineCacheQuery>{}(query); }
 };
 
 #endif /* GFX_LAYOUT_H */
