@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file map_type.h Types related to maps. */
@@ -39,20 +39,8 @@ static const uint MAX_MAP_SIZE_BITS = 12;                      ///< Maximal size
 static const uint MIN_MAP_SIZE      = 1U << MIN_MAP_SIZE_BITS; ///< Minimal map size = 64
 static const uint MAX_MAP_SIZE      = 1U << MAX_MAP_SIZE_BITS; ///< Maximal map size = 4096
 
-/**
- * Approximation of the length of a straight track, relative to a diagonal
- * track (ie the size of a tile side).
- *
- * \#defined instead of const so it can
- * stay integer. (no runtime float operations) Is this needed?
- * Watch out! There are _no_ brackets around here, to prevent intermediate
- * rounding! Be careful when using this!
- * This value should be sqrt(2)/2 ~ 0.7071
- */
-#define STRAIGHT_TRACK_LENGTH 7071/10000
-
 /** Argument for CmdLevelLand describing what to do. */
-enum LevelMode : byte {
+enum LevelMode : uint8_t {
 	LM_LEVEL, ///< Level the land.
 	LM_LOWER, ///< Lower the land.
 	LM_RAISE, ///< Raise the land.
