@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file mixer.cpp Mixing of sound samples. */
@@ -19,7 +19,7 @@
 
 struct MixerChannel {
 	/* pointer to allocated buffer memory */
-	int8_t *memory;
+	std::shared_ptr<std::vector<std::byte>> memory;
 
 	/* current position in memory */
 	uint32_t pos;
@@ -34,7 +34,10 @@ struct MixerChannel {
 	bool is16bit;
 };
 
-static std::atomic<uint8_t> _active_channels;
+using MixerChannelMask = uint8_t; ///< Type representing a bitmask of mixer channels.
+
+static std::atomic<MixerChannelMask> _active_channels;
+static std::atomic<MixerChannelMask> _stop_channels;
 static MixerChannel _channels[8];
 static uint32_t _play_rate = 11025;
 static uint32_t _max_size = UINT_MAX;
@@ -73,7 +76,7 @@ static void mix_int16(MixerChannel *sc, int16_t *buffer, uint samples, uint8_t e
 	sc->samples_left -= samples;
 	assert(samples > 0);
 
-	const T *b = (const T *)sc->memory + sc->pos;
+	const T *b = reinterpret_cast<const T *>(sc->memory->data()) + sc->pos;
 	uint32_t frac_pos = sc->frac_pos;
 	uint32_t frac_speed = sc->frac_speed;
 	int volume_left = sc->volume_left * effect_vol / 255;
@@ -100,12 +103,22 @@ static void mix_int16(MixerChannel *sc, int16_t *buffer, uint samples, uint8_t e
 	}
 
 	sc->frac_pos = frac_pos;
-	sc->pos = b - (const T *)sc->memory;
+	sc->pos = b - reinterpret_cast<const T *>(sc->memory->data());
 }
 
 static void MxCloseChannel(uint8_t channel_index)
 {
-	_active_channels.fetch_and(~(1 << channel_index), std::memory_order_release);
+	_active_channels.fetch_and(~(1U << channel_index), std::memory_order_release);
+}
+
+/**
+ * Close all mixer channels.
+ * This signals to the mixer that each channel should be closed even if it has not played all remaining samples.
+ * This is safe (and designed) to be called from the main thread.
+ */
+void MxCloseAllChannels()
+{
+	_stop_channels.fetch_or(~0, std::memory_order_release);
 }
 
 void MxMixSamples(void *buffer, uint samples)
@@ -118,7 +131,7 @@ void MxMixSamples(void *buffer, uint samples)
 	}
 
 	/* Clear the buffer */
-	memset(buffer, 0, sizeof(int16_t) * 2 * samples);
+	std::fill_n(static_cast<int16_t *>(buffer), 2 * samples, 0);
 
 	{
 		std::lock_guard<std::mutex> lock{ _music_stream_mutex };
@@ -126,6 +139,12 @@ void MxMixSamples(void *buffer, uint samples)
 		if (_music_stream) _music_stream((int16_t*)buffer, samples);
 	}
 
+	/* Check if any channels should be stopped. */
+	MixerChannelMask stop = _stop_channels.load(std::memory_order_acquire);
+	for (uint8_t idx : SetBitIterator(stop)) {
+		MxCloseChannel(idx);
+	}
+
 	/* Apply simple x^3 scaling to master effect volume. This increases the
 	 * perceived difference in loudness to better match expectations. effect_vol
 	 * is expected to be in the range 0-127 hence the division by 127 * 127 to
@@ -136,7 +155,7 @@ void MxMixSamples(void *buffer, uint samples)
 	                    effect_vol_setting) / (127 * 127);
 
 	/* Mix each channel */
-	uint8_t active = _active_channels.load(std::memory_order_acquire);
+	MixerChannelMask active = _active_channels.load(std::memory_order_acquire);
 	for (uint8_t idx : SetBitIterator(active)) {
 		MixerChannel *mc = &_channels[idx];
 		if (mc->is16bit) {
@@ -150,27 +169,29 @@ void MxMixSamples(void *buffer, uint samples)
 
 MixerChannel *MxAllocateChannel()
 {
-	uint8_t currently_active = _active_channels.load(std::memory_order_acquire);
-	uint8_t available = ~currently_active;
+	MixerChannelMask currently_active = _active_channels.load(std::memory_order_acquire);
+	MixerChannelMask available = ~currently_active;
 	if (available == 0) return nullptr;
 
 	uint8_t channel_index = FindFirstBit(available);
 
 	MixerChannel *mc = &_channels[channel_index];
-	free(mc->memory);
 	mc->memory = nullptr;
 	return mc;
 }
 
-void MxSetChannelRawSrc(MixerChannel *mc, int8_t *mem, size_t size, uint rate, bool is16bit)
+void MxSetChannelRawSrc(MixerChannel *mc, const std::shared_ptr<std::vector<std::byte>> &mem, uint rate, bool is16bit)
 {
 	mc->memory = mem;
 	mc->frac_pos = 0;
 	mc->pos = 0;
 
-	mc->frac_speed = (rate << 16) / _play_rate;
+	mc->frac_speed = (rate << 16U) / _play_rate;
 
+	size_t size = mc->memory->size();
 	if (is16bit) size /= 2;
+	/* Less 1 to allow for padding sample for the resampler. */
+	size -= 1;
 
 	/* adjust the magnitude to prevent overflow */
 	while (size >= _max_size) {
@@ -178,6 +199,7 @@ void MxSetChannelRawSrc(MixerChannel *mc, int8_t *mem, size_t size, uint rate, b
 		rate = (rate >> 1) + 1;
 	}
 
+	/* Scale number of samples by play rate. */
 	mc->samples_left = (uint)size * _play_rate / rate;
 	mc->is16bit = is16bit;
 }
@@ -200,7 +222,8 @@ void MxSetChannelVolume(MixerChannel *mc, uint volume, float pan)
 void MxActivateChannel(MixerChannel *mc)
 {
 	uint8_t channel_index = mc - _channels;
-	_active_channels.fetch_or((1 << channel_index), std::memory_order_release);
+	_stop_channels.fetch_and(~(1U << channel_index), std::memory_order_release);
+	_active_channels.fetch_or((1U << channel_index), std::memory_order_release);
 }
 
 /**
