@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /**
@@ -17,22 +17,22 @@
 
 struct NamePart {
 	std::string text; ///< If probability bit 7 is clear
-	byte id;          ///< If probability bit 7 is set
-	byte prob;        ///< The relative probability of the following name to appear in the bottom 7 bits.
+	uint8_t id;          ///< If probability bit 7 is set
+	uint8_t prob;        ///< The relative probability of the following name to appear in the bottom 7 bits.
 };
 
 struct NamePartList {
-	byte bitstart;  ///< Start of random seed bits to use.
-	byte bitcount;  ///< Number of bits of random seed to use.
+	uint8_t bitstart;  ///< Start of random seed bits to use.
+	uint8_t bitcount;  ///< Number of bits of random seed to use.
 	uint16_t maxprob; ///< Total probability of all parts.
 	std::vector<NamePart> parts; ///< List of parts to choose from.
 };
 
 struct TownNameStyle {
 	StringID name; ///< String ID of this town name style.
-	byte id;       ///< Index within partlist for this town name style.
+	uint8_t id;       ///< Index within partlist for this town name style.
 
-	TownNameStyle(StringID name, byte id) : name(name), id(id) { }
+	TownNameStyle(StringID name, uint8_t id) : name(name), id(id) { }
 };
 
 struct GRFTownName {
