@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file crashlog_win.cpp Implementation of a crashlogger for Windows */
@@ -212,8 +212,7 @@ static const uint MAX_FRAMES     = 64;
 		proc.pSymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_FAIL_CRITICAL_ERRORS | SYMOPT_UNDNAME);
 
 		/* Initialize starting stack frame from context record. */
-		STACKFRAME64 frame;
-		memset(&frame, 0, sizeof(frame));
+		STACKFRAME64 frame{};
 #ifdef _M_AMD64
 		frame.AddrPC.Offset = ep->ContextRecord->Rip;
 		frame.AddrFrame.Offset = ep->ContextRecord->Rbp;
@@ -232,12 +231,14 @@ static const uint MAX_FRAMES     = 64;
 		frame.AddrStack.Mode = AddrModeFlat;
 
 		/* Copy context record as StackWalk64 may modify it. */
-		CONTEXT ctx;
-		memcpy(&ctx, ep->ContextRecord, sizeof(ctx));
+		CONTEXT ctx = *ep->ContextRecord;
 
-		/* Allocate space for symbol info. */
-		char sym_info_raw[sizeof(IMAGEHLP_SYMBOL64) + MAX_SYMBOL_LEN - 1];
-		IMAGEHLP_SYMBOL64 *sym_info = (IMAGEHLP_SYMBOL64*)sym_info_raw;
+		/* Allocate space for symbol info.
+		 * The total initialised size must be sufficient for a null-terminating char at sym_info->Name[sym_info->MaxNameLength],
+		 * SymGetSymFromAddr64 is not required to write a null-terminating char.
+		 * sizeof(IMAGEHLP_SYMBOL64) includes at least one char of the Name buffer. */
+		std::array<char, sizeof(IMAGEHLP_SYMBOL64) + MAX_SYMBOL_LEN> sym_info_raw{};
+		IMAGEHLP_SYMBOL64 *sym_info = reinterpret_cast<IMAGEHLP_SYMBOL64*>(sym_info_raw.data());
 		sym_info->SizeOfStruct = sizeof(IMAGEHLP_SYMBOL64);
 		sym_info->MaxNameLength = MAX_SYMBOL_LEN;
 
@@ -257,7 +258,7 @@ static const uint MAX_FRAMES     = 64;
 			}
 
 			/* Get module name. */
-			const char *mod_name = "???";
+			std::string_view mod_name = "???";
 
 			IMAGEHLP_MODULE64 module;
 			module.SizeOfStruct = sizeof(module);
@@ -266,18 +267,18 @@ static const uint MAX_FRAMES     = 64;
 			}
 
 			/* Print module and instruction pointer. */
-			std::string message = fmt::format("{:20s} {:X}",  mod_name, frame.AddrPC.Offset);
+			std::string message = fmt::format("{:20s} {:X}", mod_name, frame.AddrPC.Offset);
 
 			/* Get symbol name and line info if possible. */
 			DWORD64 offset;
 			if (proc.pSymGetSymFromAddr64(hCur, frame.AddrPC.Offset, &offset, sym_info)) {
-				message += fmt::format(" {} + {}", sym_info->Name, offset);
+				format_append(message, " {} + {}", sym_info->Name, offset);
 
 				DWORD line_offs;
 				IMAGEHLP_LINE64 line;
 				line.SizeOfStruct = sizeof(IMAGEHLP_LINE64);
 				if (proc.pSymGetLineFromAddr64(hCur, frame.AddrPC.Offset, &line_offs, &line)) {
-					message += fmt::format(" ({}:{})", line.FileName, line.LineNumber);
+					format_append(message, " ({}:{})", line.FileName, line.LineNumber);
 				}
 			}
 
@@ -469,7 +470,7 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 			std::string crashlog = CrashLogWindows::current->survey.dump(4);
 			size_t crashlog_length = crashlog.size() + 1;
 			/* Reserve extra space for LF to CRLF conversion. */
-			crashlog_length += std::count(crashlog.begin(), crashlog.end(), '\n');
+			crashlog_length += std::ranges::count(crashlog, '\n');
 
 			const size_t filename_count = 4;
 			const size_t filename_buf_length = MAX_PATH + 1;
@@ -491,27 +492,27 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 			char *crashlog_dos_nl = reinterpret_cast<char *>(filename_buf + filename_buf_length * filename_count);
 
 			/* Convert unix -> dos newlines because the edit box only supports that properly. */
-			const char *crashlog_unix_nl = crashlog.data();
-			char *p = crashlog_dos_nl;
-			char32_t c;
-			while ((c = Utf8Consume(&crashlog_unix_nl))) {
-				if (c == '\n') p += Utf8Encode(p, '\r');
-				p += Utf8Encode(p, c);
+			{
+				char *p = crashlog_dos_nl;
+				for (char c : crashlog) {
+					if (c == '\n') *(p++) = '\r';
+					*(p++) = c;
+				}
+				*p = '\0';
 			}
-			*p = '\0';
 
 			_snwprintf(
 				crash_desc_buf,
 				crash_desc_buf_length,
 				_crash_desc,
-				convert_to_fs(CrashLogWindows::current->crashlog_filename,   filename_buf + filename_buf_length * 0, filename_buf_length),
-				convert_to_fs(CrashLogWindows::current->crashdump_filename,  filename_buf + filename_buf_length * 1, filename_buf_length),
-				convert_to_fs(CrashLogWindows::current->savegame_filename,   filename_buf + filename_buf_length * 2, filename_buf_length),
-				convert_to_fs(CrashLogWindows::current->screenshot_filename, filename_buf + filename_buf_length * 3, filename_buf_length)
+				convert_to_fs(CrashLogWindows::current->crashlog_filename,   {filename_buf + filename_buf_length * 0, filename_buf_length}),
+				convert_to_fs(CrashLogWindows::current->crashdump_filename,  {filename_buf + filename_buf_length * 1, filename_buf_length}),
+				convert_to_fs(CrashLogWindows::current->savegame_filename,   {filename_buf + filename_buf_length * 2, filename_buf_length}),
+				convert_to_fs(CrashLogWindows::current->screenshot_filename, {filename_buf + filename_buf_length * 3, filename_buf_length})
 			);
 
 			SetDlgItemText(wnd, 10, crash_desc_buf);
-			SetDlgItemText(wnd, 11, convert_to_fs(crashlog_dos_nl, crashlog_buf, crashlog_length));
+			SetDlgItemText(wnd, 11, convert_to_fs(crashlog_dos_nl, {crashlog_buf, crashlog_length}));
 			SendDlgItemMessage(wnd, 11, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), FALSE);
 			SetWndSize(wnd, -1);
 		} return TRUE;
