@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file rail_map.h Hides the direct accesses to the map array with map accessors */
@@ -20,10 +20,10 @@
 
 
 /** Different types of Rail-related tiles */
-enum RailTileType {
-	RAIL_TILE_NORMAL   = 0, ///< Normal rail tile without signals
-	RAIL_TILE_SIGNALS  = 1, ///< Normal rail tile with signals
-	RAIL_TILE_DEPOT    = 3, ///< Depot (one entrance)
+enum class RailTileType : uint8_t {
+	Normal = 0, ///< Normal rail tile without signals
+	Signals = 1, ///< Normal rail tile with signals
+	Depot = 3, ///< Depot (one entrance)
 };
 
 /**
@@ -33,23 +33,23 @@ enum RailTileType {
  * @pre IsTileType(t, MP_RAILWAY)
  * @return the RailTileType
  */
-debug_inline static RailTileType GetRailTileType(Tile t)
+[[debug_inline]] inline static RailTileType GetRailTileType(Tile t)
 {
 	assert(IsTileType(t, MP_RAILWAY));
-	return (RailTileType)GB(t.m5(), 6, 2);
+	return static_cast<RailTileType>(GB(t.m5(), 6, 2));
 }
 
 /**
  * Returns whether this is plain rails, with or without signals. Iow, if this
- * tiles RailTileType is RAIL_TILE_NORMAL or RAIL_TILE_SIGNALS.
+ * tiles RailTileType is RailTileType::Normal or RailTileType::Signals.
  * @param t the tile to get the information from
  * @pre IsTileType(t, MP_RAILWAY)
  * @return true if and only if the tile is normal rail (with or without signals)
  */
-debug_inline static bool IsPlainRail(Tile t)
+[[debug_inline]] inline static bool IsPlainRail(Tile t)
 {
 	RailTileType rtt = GetRailTileType(t);
-	return rtt == RAIL_TILE_NORMAL || rtt == RAIL_TILE_SIGNALS;
+	return rtt == RailTileType::Normal || rtt == RailTileType::Signals;
 }
 
 /**
@@ -57,7 +57,7 @@ debug_inline static bool IsPlainRail(Tile t)
  * @param t the tile to get the information from
  * @return true if and only if the tile is normal rail (with or without signals)
  */
-debug_inline static bool IsPlainRailTile(Tile t)
+[[debug_inline]] inline static bool IsPlainRailTile(Tile t)
 {
 	return IsTileType(t, MP_RAILWAY) && IsPlainRail(t);
 }
@@ -71,7 +71,7 @@ debug_inline static bool IsPlainRailTile(Tile t)
  */
 inline bool HasSignals(Tile t)
 {
-	return GetRailTileType(t) == RAIL_TILE_SIGNALS;
+	return GetRailTileType(t) == RailTileType::Signals;
 }
 
 /**
@@ -83,7 +83,7 @@ inline bool HasSignals(Tile t)
 inline void SetHasSignals(Tile tile, bool signals)
 {
 	assert(IsPlainRailTile(tile));
-	SB(tile.m5(), 6, 1, signals);
+	SB(tile.m5(), 6, 2, to_underlying(signals ? RailTileType::Signals : RailTileType::Normal));
 }
 
 /**
@@ -92,9 +92,9 @@ inline void SetHasSignals(Tile tile, bool signals)
  * @pre IsTileType(t, MP_RAILWAY)
  * @return true if and only if the tile is a rail depot
  */
-debug_inline static bool IsRailDepot(Tile t)
+[[debug_inline]] inline static bool IsRailDepot(Tile t)
 {
-	return GetRailTileType(t) == RAIL_TILE_DEPOT;
+	return GetRailTileType(t) == RailTileType::Depot;
 }
 
 /**
@@ -102,7 +102,7 @@ debug_inline static bool IsRailDepot(Tile t)
  * @param t the tile to get the information from
  * @return true if and only if the tile is a rail depot
  */
-debug_inline static bool IsRailDepotTile(Tile t)
+[[debug_inline]] inline static bool IsRailDepotTile(Tile t)
 {
 	return IsTileType(t, MP_RAILWAY) && IsRailDepot(t);
 }
@@ -194,7 +194,7 @@ inline Track GetRailDepotTrack(Tile t)
 inline TrackBits GetRailReservationTrackBits(Tile t)
 {
 	assert(IsPlainRailTile(t));
-	byte track_b = GB(t.m2(), 8, 3);
+	uint8_t track_b = GB(t.m2(), 8, 3);
 	Track track = (Track)(track_b - 1);    // map array saves Track+1
 	if (track_b == 0) return TRACK_BIT_NONE;
 	return (TrackBits)(TrackToTrackBits(track) | (HasBit(t.m2(), 11) ? TrackToTrackBits(TrackToOppositeTrack(track)) : 0));
@@ -209,18 +209,17 @@ inline TrackBits GetRailReservationTrackBits(Tile t)
 inline void SetTrackReservation(Tile t, TrackBits b)
 {
 	assert(IsPlainRailTile(t));
-	assert(b != INVALID_TRACK_BIT);
 	assert(!TracksOverlap(b));
 	Track track = RemoveFirstTrack(&b);
 	SB(t.m2(), 8, 3, track == INVALID_TRACK ? 0 : track + 1);
-	SB(t.m2(), 11, 1, (byte)(b != TRACK_BIT_NONE));
+	AssignBit(t.m2(), 11, b != TRACK_BIT_NONE);
 }
 
 /**
  * Try to reserve a specific track on a tile
- * @pre IsPlainRailTile(t) && HasTrack(tile, t)
+ * @pre IsPlainRailTile(tile) && HasTrack(tile, t)
  * @param tile the tile
- * @param t the rack to reserve
+ * @param t the track to reserve
  * @return true if successful
  */
 inline bool TryReserveTrack(Tile tile, Track t)
@@ -237,7 +236,7 @@ inline bool TryReserveTrack(Tile tile, Track t)
 
 /**
  * Lift the reservation of a specific track on a tile
- * @pre IsPlainRailTile(t) && HasTrack(tile, t)
+ * @pre IsPlainRailTile(tile) && HasTrack(tile, t)
  * @param tile the tile
  * @param t the track to free
  */
@@ -270,7 +269,7 @@ inline bool HasDepotReservation(Tile t)
 inline void SetDepotReservation(Tile t, bool b)
 {
 	assert(IsRailDepot(t));
-	SB(t.m5(), 4, 1, (byte)b);
+	AssignBit(t.m5(), 4, b);
 }
 
 /**
@@ -292,15 +291,15 @@ inline bool IsPbsSignal(SignalType s)
 
 inline SignalType GetSignalType(Tile t, Track track)
 {
-	assert(GetRailTileType(t) == RAIL_TILE_SIGNALS);
-	byte pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 4 : 0;
+	assert(GetRailTileType(t) == RailTileType::Signals);
+	uint8_t pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 4 : 0;
 	return (SignalType)GB(t.m2(), pos, 3);
 }
 
 inline void SetSignalType(Tile t, Track track, SignalType s)
 {
-	assert(GetRailTileType(t) == RAIL_TILE_SIGNALS);
-	byte pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 4 : 0;
+	assert(GetRailTileType(t) == RailTileType::Signals);
+	uint8_t pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 4 : 0;
 	SB(t.m2(), pos, 3, s);
 	if (track == INVALID_TRACK) SB(t.m2(), 4, 3, s);
 }
@@ -323,8 +322,8 @@ inline bool IsOnewaySignal(Tile t, Track track)
 
 inline void CycleSignalSide(Tile t, Track track)
 {
-	byte sig;
-	byte pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 4 : 6;
+	uint8_t sig;
+	uint8_t pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 4 : 6;
 
 	sig = GB(t.m3(), pos, 2);
 	if (--sig == 0) sig = IsPbsSignal(GetSignalType(t, track)) ? 2 : 3;
@@ -333,13 +332,13 @@ inline void CycleSignalSide(Tile t, Track track)
 
 inline SignalVariant GetSignalVariant(Tile t, Track track)
 {
-	byte pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 7 : 3;
+	uint8_t pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 7 : 3;
 	return (SignalVariant)GB(t.m2(), pos, 1);
 }
 
 inline void SetSignalVariant(Tile t, Track track, SignalVariant v)
 {
-	byte pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 7 : 3;
+	uint8_t pos = (track == TRACK_LOWER || track == TRACK_RIGHT) ? 7 : 3;
 	SB(t.m2(), pos, 1, v);
 	if (track == INVALID_TRACK) SB(t.m2(), 7, 1, v);
 }
@@ -370,7 +369,7 @@ inline uint GetSignalStates(Tile tile)
  * @param signalbit the signal
  * @return the state of the signal
  */
-inline SignalState GetSingleSignalState(Tile t, byte signalbit)
+inline SignalState GetSingleSignalState(Tile t, uint8_t signalbit)
 {
 	return (SignalState)HasBit(GetSignalStates(t), signalbit);
 }
@@ -401,7 +400,7 @@ inline uint GetPresentSignals(Tile tile)
  * @param signalbit the signal
  * @return true if and only if the signal is present
  */
-inline bool IsSignalPresent(Tile t, byte signalbit)
+inline bool IsSignalPresent(Tile t, uint8_t signalbit)
 {
 	return HasBit(GetPresentSignals(t), signalbit);
 }
@@ -413,7 +412,7 @@ inline bool IsSignalPresent(Tile t, byte signalbit)
 inline bool HasSignalOnTrack(Tile tile, Track track)
 {
 	assert(IsValidTrack(track));
-	return GetRailTileType(tile) == RAIL_TILE_SIGNALS && (GetPresentSignals(tile) & SignalOnTrack(track)) != 0;
+	return GetRailTileType(tile) == RailTileType::Signals && (GetPresentSignals(tile) & SignalOnTrack(track)) != 0;
 }
 
 /**
@@ -426,7 +425,7 @@ inline bool HasSignalOnTrack(Tile tile, Track track)
 inline bool HasSignalOnTrackdir(Tile tile, Trackdir trackdir)
 {
 	assert (IsValidTrackdir(trackdir));
-	return GetRailTileType(tile) == RAIL_TILE_SIGNALS && GetPresentSignals(tile) & SignalAlongTrackdir(trackdir);
+	return GetRailTileType(tile) == RailTileType::Signals && GetPresentSignals(tile) & SignalAlongTrackdir(trackdir);
 }
 
 /**
@@ -482,37 +481,37 @@ inline bool HasOnewaySignalBlockingTrackdir(Tile tile, Trackdir td)
 RailType GetTileRailType(Tile tile);
 
 /** The ground 'under' the rail */
-enum RailGroundType {
-	RAIL_GROUND_BARREN       =  0, ///< Nothing (dirt)
-	RAIL_GROUND_GRASS        =  1, ///< Grassy
-	RAIL_GROUND_FENCE_NW     =  2, ///< Grass with a fence at the NW edge
-	RAIL_GROUND_FENCE_SE     =  3, ///< Grass with a fence at the SE edge
-	RAIL_GROUND_FENCE_SENW   =  4, ///< Grass with a fence at the NW and SE edges
-	RAIL_GROUND_FENCE_NE     =  5, ///< Grass with a fence at the NE edge
-	RAIL_GROUND_FENCE_SW     =  6, ///< Grass with a fence at the SW edge
-	RAIL_GROUND_FENCE_NESW   =  7, ///< Grass with a fence at the NE and SW edges
-	RAIL_GROUND_FENCE_VERT1  =  8, ///< Grass with a fence at the eastern side
-	RAIL_GROUND_FENCE_VERT2  =  9, ///< Grass with a fence at the western side
-	RAIL_GROUND_FENCE_HORIZ1 = 10, ///< Grass with a fence at the southern side
-	RAIL_GROUND_FENCE_HORIZ2 = 11, ///< Grass with a fence at the northern side
-	RAIL_GROUND_ICE_DESERT   = 12, ///< Icy or sandy
-	RAIL_GROUND_WATER        = 13, ///< Grass with a fence and shore or water on the free halftile
-	RAIL_GROUND_HALF_SNOW    = 14, ///< Snow only on higher part of slope (steep or one corner raised)
+enum class RailGroundType : uint8_t {
+	Barren = 0, ///< Nothing (dirt)
+	Grass = 1, ///< Grassy
+	FenceNW = 2, ///< Grass with a fence at the NW edge
+	FenceSE = 3, ///< Grass with a fence at the SE edge
+	FenceSENW = 4, ///< Grass with a fence at the NW and SE edges
+	FenceNE = 5, ///< Grass with a fence at the NE edge
+	FenceSW = 6, ///< Grass with a fence at the SW edge
+	FenceNESW = 7, ///< Grass with a fence at the NE and SW edges
+	FenceVert1 = 8, ///< Grass with a fence at the eastern side
+	FenceVert2 = 9, ///< Grass with a fence at the western side
+	FenceHoriz1 = 10, ///< Grass with a fence at the southern side
+	FenceHoriz2 = 11, ///< Grass with a fence at the northern side
+	SnowOrDesert = 12, ///< Icy or sandy
+	HalfTileWater = 13, ///< Grass with a fence and shore or water on the free halftile
+	HalfTileSnow = 14, ///< Snow only on higher part of slope (steep or one corner raised)
 };
 
 inline void SetRailGroundType(Tile t, RailGroundType rgt)
 {
-	SB(t.m4(), 0, 4, rgt);
+	SB(t.m4(), 0, 4, to_underlying(rgt));
 }
 
 inline RailGroundType GetRailGroundType(Tile t)
 {
-	return (RailGroundType)GB(t.m4(), 0, 4);
+	return static_cast<RailGroundType>(GB(t.m4(), 0, 4));
 }
 
 inline bool IsSnowRailGround(Tile t)
 {
-	return GetRailGroundType(t) == RAIL_GROUND_ICE_DESERT;
+	return GetRailGroundType(t) == RailGroundType::SnowOrDesert;
 }
 
 
@@ -524,8 +523,8 @@ inline void MakeRailNormal(Tile t, Owner o, TrackBits b, RailType r)
 	t.m2() = 0;
 	t.m3() = 0;
 	t.m4() = 0;
-	t.m5() = RAIL_TILE_NORMAL << 6 | b;
-	SB(t.m6(), 2, 4, 0);
+	t.m5() = to_underlying(RailTileType::Normal) << 6 | b;
+	SB(t.m6(), 2, 6, 0);
 	t.m7() = 0;
 	t.m8() = r;
 }
@@ -554,11 +553,11 @@ inline void MakeRailDepot(Tile tile, Owner owner, DepotID depot_id, DiagDirectio
 	SetTileType(tile, MP_RAILWAY);
 	SetTileOwner(tile, owner);
 	SetDockingTile(tile, false);
-	tile.m2() = depot_id;
+	tile.m2() = depot_id.base();
 	tile.m3() = 0;
 	tile.m4() = 0;
-	tile.m5() = RAIL_TILE_DEPOT << 6 | dir;
-	SB(tile.m6(), 2, 4, 0);
+	tile.m5() = to_underlying(RailTileType::Depot) << 6 | dir;
+	SB(tile.m6(), 2, 6, 0);
 	tile.m7() = 0;
 	tile.m8() = rail_type;
 }
