@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /**
@@ -10,49 +10,71 @@
  *
  * Unsafe methods are, for example, strndup and strncpy because they may leave the
  * string without a null termination, but also strdup and strndup because they can
- * return nullptr and then all strdups would need to be guarded against that instead
- * of using the current MallocT/ReallocT/CallocT technique of just giving the user
- * an error that too much memory was used instead of spreading that code though
- * the whole code base.
+ * return nullptr and then all strdups would need to be guarded against.
  */
 
 #ifndef SAFEGUARDS_H
 #define SAFEGUARDS_H
 
-/* Use MallocT instead. */
+/* Use std::vector/std::unique_ptr/new instead. */
 #define malloc    SAFEGUARD_DO_NOT_USE_THIS_METHOD
-
-/* Use MallocT instead. */
 #define calloc    SAFEGUARD_DO_NOT_USE_THIS_METHOD
-
-/* Use ReallocT instead. */
 #define realloc   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
-/* Use std::string instead. */
+/* Use std::string/std::string_view instead. */
 #define strdup    SAFEGUARD_DO_NOT_USE_THIS_METHOD
 #define strndup   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
-/* Use strecpy instead. */
 #define strcpy    SAFEGUARD_DO_NOT_USE_THIS_METHOD
 #define strncpy   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
-/* Use std::string concatenation/fmt::format instead. */
 #define strcat    SAFEGUARD_DO_NOT_USE_THIS_METHOD
 #define strncat   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
-/* Use fmt::format instead. */
 #define sprintf   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 #define snprintf  SAFEGUARD_DO_NOT_USE_THIS_METHOD
-
-/* Use fmt::format instead. */
 #define vsprintf  SAFEGUARD_DO_NOT_USE_THIS_METHOD
 #define vsnprintf SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
+#define strcmp SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define strncmp SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#ifdef strcasecmp
+#undef strcasecmp
+#endif
+#define strcasecmp SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#ifdef stricmp
+#undef stricmp
+#endif
+#define stricmp SAFEGUARD_DO_NOT_USE_THIS_METHOD
+
+#define memcmp SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define memcpy SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define memmove SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define memset SAFEGUARD_DO_NOT_USE_THIS_METHOD
+
 /* Use fgets instead. */
 #define gets      SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
-/* No clear replacement. */
+/* Use StringConsumer instead. */
 #define strtok    SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define sscanf    SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define from_string SAFEGUARD_DO_NOT_USE_THIS_METHOD
+
+/* Use ParseInteger or StringConsumer instead. */
+#define atoi      SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define atol      SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define atoll     SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define strtol    SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define strtoll   SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define strtoul   SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define strtoull  SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stoi      SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stol      SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stoll     SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stoul     SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stoull    SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stoimax   SAFEGUARD_DO_NOT_USE_THIS_METHOD
+#define stoumax   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
 /* Use fmt::print instead. */
 #define printf    SAFEGUARD_DO_NOT_USE_THIS_METHOD
@@ -61,6 +83,9 @@
 #define fputs     SAFEGUARD_DO_NOT_USE_THIS_METHOD
 #define putchar   SAFEGUARD_DO_NOT_USE_THIS_METHOD
 
+/* Use fmt::format instead */
+#define to_string SAFEGUARD_DO_NOT_USE_THIS_METHOD
+
 /* Use our own templated implementation instead of a macro or function with only one type. */
 #ifdef min
 #undef min
