@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file script_bridgelist.hpp List all the bridges. */
@@ -32,7 +32,7 @@ public:
 	/**
 	 * @param length The length of the bridge you want to build.
 	 */
-	ScriptBridgeList_Length(uint length);
+	ScriptBridgeList_Length(SQInteger length);
 };
 
 #endif /* SCRIPT_BRIDGELIST_HPP */
