@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file script_event.hpp Everything to handle events from the game. */
@@ -57,8 +57,11 @@ public:
 		ET_STORYPAGE_BUTTON_CLICK,
 		ET_STORYPAGE_TILE_SELECT,
 		ET_STORYPAGE_VEHICLE_SELECT,
+		ET_COMPANY_RENAMED,
+		ET_PRESIDENT_RENAMED,
 	};
 
+#ifndef DOXYGEN_API
 	/**
 	 * Constructor of ScriptEvent, to get the type of event.
 	 * @param type The type of event to construct.
@@ -66,6 +69,7 @@ public:
 	ScriptEvent(ScriptEvent::ScriptEventType type) :
 		type(type)
 	{}
+#endif /* DOXYGEN_API */
 
 	/**
 	 * Get the event-type.
