@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file script_gamesettings.cpp Implementation of ScriptGameSettings. */
@@ -39,7 +39,7 @@
 	const SettingDesc *sd = GetSettingFromName(setting);
 	assert(sd != nullptr);
 
-	if ((sd->flags & SF_NO_NETWORK_SYNC) != 0) return false;
+	if ((sd->flags.Test(SettingFlag::NoNetworkSync))) return false;
 
 	value = Clamp<SQInteger>(value, INT32_MIN, INT32_MAX);
 
