@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file script_industrylist.hpp List all the industries. */
@@ -25,20 +25,23 @@ public:
 	/**
 	 * Apply a filter when building the list.
 	 * @param filter_function The function which will be doing the filtering.
-	 * @param params The params to give to the filters (minus the first param,
+	 * @param ... The params to give to the filters (minus the first param,
 	 *  which is always the index-value).
 	 * @note You can write your own filters and use them. Just remember that
 	 *  the first parameter should be the index-value, and it should return
 	 *  a bool.
 	 * @note Example:
-	 *  ScriptIndustryList(ScriptIndustry.HasDock);
+	 * @code
+	 *  local water_industries = ScriptIndustryList(ScriptIndustry.HasDock);
+	 *
 	 *  function IsType(industry_id, type)
 	 *  {
 	 *    return ScriptIndustry.GetIndustryType(industry_id) == type;
 	 *  }
-	 *  ScriptIndustryList(IsType, 0);
+	 *  local industries = ScriptIndustryList(IsType, 0);
+	 * @endcode
 	 */
-	ScriptIndustryList(void *filter_function, int params, ...);
+	ScriptIndustryList(function filter_function, ...);
 #else
 	/**
 	 * The constructor wrapper from Squirrel.
@@ -55,9 +58,9 @@ public:
 class ScriptIndustryList_CargoAccepting : public ScriptList {
 public:
 	/**
-	 * @param cargo_id The cargo this industry should accept.
+	 * @param cargo_type The cargo this industry should accept.
 	 */
-	ScriptIndustryList_CargoAccepting(CargoID cargo_id);
+	ScriptIndustryList_CargoAccepting(CargoType cargo_type);
 };
 
 /**
@@ -69,9 +72,9 @@ public:
 class ScriptIndustryList_CargoProducing : public ScriptList {
 public:
 	/**
-	 * @param cargo_id The cargo this industry should produce.
+	 * @param cargo_type The cargo this industry should produce.
 	 */
-	ScriptIndustryList_CargoProducing(CargoID cargo_id);
+	ScriptIndustryList_CargoProducing(CargoType cargo_type);
 };
 
 #endif /* SCRIPT_INDUSTRYLIST_HPP */
