@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file settings_type.h Types related to global configuration settings. */
@@ -10,6 +10,7 @@
 #ifndef SETTINGS_TYPE_H
 #define SETTINGS_TYPE_H
 
+#include "command_type.h"
 #include "timer/timer_game_calendar.h"
 #include "economy_type.h"
 #include "town_type.h"
@@ -35,7 +36,7 @@ const size_t MAX_SLE_INT32 = INT32_MAX;
 const size_t MAX_SLE_INT = INT_MAX;
 
 /** Settings profiles and highscore tables. */
-enum SettingsProfile {
+enum SettingsProfile : uint8_t {
 	SP_BEGIN = 0,
 	SP_EASY = SP_BEGIN,                       ///< Easy difficulty.
 	SP_MEDIUM,                                ///< Medium difficulty.
@@ -51,7 +52,7 @@ enum SettingsProfile {
 };
 
 /** Available industry map generation densities. */
-enum IndustryDensity {
+enum IndustryDensity : uint8_t {
 	ID_FUND_ONLY, ///< The game does not build industries.
 	ID_MINIMAL,   ///< Start with just the industries that must be present.
 	ID_VERY_LOW,  ///< Very few industries at game start.
@@ -64,6 +65,13 @@ enum IndustryDensity {
 	ID_END,       ///< Number of industry density settings.
 };
 
+/** Possible options for the Borders pulldown in the Genworld GUI. */
+enum BorderFlagPresets : uint8_t {
+	BFP_RANDOM = 0,
+	BFP_MANUAL,
+	BFP_INFINITE_WATER,
+};
+
 /** Possible values for the "timekeeping_units" setting. */
 enum TimekeepingUnits : uint8_t {
 	TKU_CALENDAR = 0,
@@ -91,34 +99,52 @@ enum RightClickClose : uint8_t {
 	RCC_YES_EXCEPT_STICKY,
 };
 
+/**
+ * List of tree placer algorithm.
+ *
+ * This enumeration defines all possible tree placer algorithm in the game.
+ */
+enum TreePlacer : uint8_t {
+	TP_NONE,     ///< No tree placer algorithm
+	TP_ORIGINAL, ///< The original algorithm
+	TP_IMPROVED, ///< A 'improved' algorithm
+};
+
+/** Possible values for "place_houses" setting. */
+enum PlaceHouses : uint8_t {
+	PH_FORBIDDEN = 0,
+	PH_ALLOWED,
+	PH_ALLOWED_CONSTRUCTED,
+};
+
 /** Settings related to the difficulty of the game */
 struct DifficultySettings {
-	byte   competitor_start_time;            ///< Unused value, used to load old savegames.
-	byte   competitor_intelligence;          ///< Unused value, used to load old savegames.
+	uint8_t competitor_start_time;            ///< Unused value, used to load old savegames.
+	uint8_t competitor_intelligence;          ///< Unused value, used to load old savegames.
 
-	byte   max_no_competitors;               ///< the number of competitors (AIs)
+	uint8_t max_no_competitors;               ///< the number of competitors (AIs)
 	uint16_t competitors_interval;             ///< the interval (in minutes) between adding competitors
-	byte   number_towns;                     ///< the amount of towns
-	byte   industry_density;                 ///< The industry density. @see IndustryDensity
+	uint8_t number_towns;                     ///< the amount of towns
+	uint8_t industry_density;                 ///< The industry density. @see IndustryDensity
 	uint32_t max_loan;                         ///< the maximum initial loan
-	byte   initial_interest;                 ///< amount of interest (to pay over the loan)
-	byte   vehicle_costs;                    ///< amount of money spent on vehicle running cost
-	byte   competitor_speed;                 ///< the speed at which the AI builds
-	byte   vehicle_breakdowns;               ///< likelihood of vehicles breaking down
-	byte   subsidy_multiplier;               ///< payment multiplier for subsidized deliveries
+	uint8_t initial_interest;                 ///< amount of interest (to pay over the loan)
+	uint8_t vehicle_costs;                    ///< amount of money spent on vehicle running cost
+	uint8_t competitor_speed;                 ///< the speed at which the AI builds
+	uint8_t vehicle_breakdowns;               ///< likelihood of vehicles breaking down
+	uint8_t subsidy_multiplier;               ///< payment multiplier for subsidized deliveries
 	uint16_t subsidy_duration;                 ///< duration of subsidies
-	byte   construction_cost;                ///< how expensive is building
-	byte   terrain_type;                     ///< the mountainousness of the landscape
-	byte   quantity_sea_lakes;               ///< the amount of seas/lakes
+	uint8_t construction_cost;                ///< how expensive is building
+	uint8_t terrain_type;                     ///< the mountainousness of the landscape
+	uint8_t quantity_sea_lakes;               ///< the amount of seas/lakes
 	bool   economy;                          ///< how volatile is the economy
 	bool   line_reverse_mode;                ///< reversing at stations or not
 	bool   disasters;                        ///< are disasters enabled
-	byte   town_council_tolerance;           ///< minimum required town ratings to be allowed to demolish stuff
+	uint8_t town_council_tolerance;           ///< minimum required town ratings to be allowed to demolish stuff
 	bool   infinite_money;                   ///< whether spending money despite negative balance is allowed
 };
 
 /** Settings relating to viewport/smallmap scrolling. */
-enum ViewportScrollMode {
+enum ViewportScrollMode : uint8_t {
 	VSM_VIEWPORT_RMB_FIXED, ///< Viewport moves with mouse movement on holding right mouse button, cursor position is fixed.
 	VSM_MAP_RMB_FIXED,      ///< Map moves with mouse movement on holding right mouse button, cursor position is fixed.
 	VSM_MAP_RMB,            ///< Map moves with mouse movement on holding right mouse button, cursor moves.
@@ -126,18 +152,26 @@ enum ViewportScrollMode {
 	VSM_END,                ///< Number of scroll mode settings.
 };
 
+/** Settings related to scroll wheel behavior. */
+enum ScrollWheelScrollingSetting : uint8_t {
+	SWS_ZOOM_MAP = 0,       ///< Scroll wheel zooms the map.
+	SWS_SCROLL_MAP = 1,     ///< Scroll wheel scrolls the map.
+	SWS_OFF = 2             ///< Scroll wheel has no effect.
+};
+
 /** Settings related to the GUI and other stuff that is not saved in the savegame. */
 struct GUISettings {
 	bool   sg_full_load_any;                 ///< new full load calculation, any cargo must be full read from pre v93 savegames
 	bool   lost_vehicle_warn;                ///< if a vehicle can't find its destination, show a warning
 	uint8_t  order_review_system;              ///< perform order reviews on vehicles
 	bool   vehicle_income_warn;              ///< if a vehicle isn't generating income, show a warning
+	bool   old_vehicle_warn;                 ///< if a vehicle is getting old, show a warning
 	bool   show_finances;                    ///< show finances at end of year
 	bool   sg_new_nonstop;                   ///< ttdpatch compatible nonstop handling read from pre v93 savegames
 	bool   new_nonstop;                      ///< ttdpatch compatible nonstop handling
-	uint8_t  stop_location;                    ///< what is the default stop location of trains?
+	OrderStopLocation stop_location; ///< what is the default stop location of trains?
 	uint8_t  auto_scrolling;                   ///< scroll when moving mouse to the edge (see #ViewportAutoscrolling)
-	byte   errmsg_duration;                  ///< duration of error message
+	uint8_t errmsg_duration;                  ///< duration of error message
 	uint16_t hover_delay_ms;                   ///< time required to activate a hover event, in milliseconds
 	bool   link_terraform_toolbar;           ///< display terraform toolbar when displaying rail, road, water and airport toolbars
 	uint8_t  smallmap_land_colour;             ///< colour used for land and heightmap at the smallmap
@@ -145,7 +179,7 @@ struct GUISettings {
 	uint8_t  scroll_mode;                      ///< viewport scroll mode
 	bool   smooth_scroll;                    ///< smooth scroll viewports
 	bool   measure_tooltip;                  ///< show a permanent tooltip when dragging tools
-	byte   liveries;                         ///< options for displaying company liveries, 0=none, 1=self, 2=all
+	uint8_t liveries;                         ///< options for displaying company liveries, 0=none, 1=self, 2=all
 	bool   prefer_teamchat;                  ///< choose the chat message target with \<ENTER\>, true=all clients, false=your team
 	uint8_t  advanced_vehicle_list;            ///< use the "advanced" vehicle list
 	uint8_t  loading_indicators;               ///< show loading indicators
@@ -163,13 +197,14 @@ struct GUISettings {
 	bool   autosave_on_exit;                 ///< save an autosave when you quit the game, but do not ask "Do you really want to quit?"
 	bool   autosave_on_network_disconnect;   ///< save an autosave when you get disconnected from a network game with an error?
 	uint8_t  date_format_in_default_names;     ///< should the default savegame/screenshot name use long dates (31th Dec 2008), short dates (31-12-2008) or ISO dates (2008-12-31)
-	byte   max_num_autosaves;                ///< controls how many autosavegames are made before the game starts to overwrite (names them 0 to max_num_autosaves - 1)
+	uint8_t max_num_autosaves;                ///< controls how many autosavegames are made before the game starts to overwrite (names them 0 to max_num_autosaves - 1)
 	bool   population_in_label;              ///< show the population of a town in its label?
 	uint8_t  right_mouse_btn_emulation;        ///< should we emulate right mouse clicking?
 	uint8_t  scrollwheel_scrolling;            ///< scrolling using the scroll wheel?
 	uint8_t  scrollwheel_multiplier;           ///< how much 'wheel' per incoming event from the OS?
 	bool   timetable_arrival_departure;      ///< show arrivals and departures in vehicle timetables
 	RightClickClose  right_click_wnd_close;  ///< close window with right click
+	bool   toolbar_dropdown_autoselect;      ///< should toolbar dropdown buttons autoselect when releasing the mouse button
 	bool   pause_on_newgame;                 ///< whether to start new games paused or not
 	SignalGUISettings signal_gui_mode;       ///< select which signal types are shown in the signal GUI
 	SignalCycleSettings cycle_signal_types;  ///< Which signal types to cycle with the build signal tool.
@@ -178,23 +213,23 @@ struct GUISettings {
 	TimetableMode timetable_mode;            ///< Time units for timetables: days, seconds, or ticks
 	bool   quick_goto;                       ///< Allow quick access to 'goto button' in vehicle orders window
 	bool   auto_euro;                        ///< automatically switch to euro in 2002
-	byte   drag_signals_density;             ///< many signals density
+	uint8_t drag_signals_density;             ///< many signals density
 	bool   drag_signals_fixed_distance;      ///< keep fixed distance between signals when dragging
 	TimerGameCalendar::Year semaphore_build_before; ///< build semaphore signals automatically before this year
-	byte   news_message_timeout;             ///< how much longer than the news message "age" should we keep the message in the history
+	uint8_t news_message_timeout;             ///< how much longer than the news message "age" should we keep the message in the history
 	bool   show_track_reservation;           ///< highlight reserved tracks.
-	byte   station_numtracks;                ///< the number of platforms to default on for rail stations
-	byte   station_platlength;               ///< the platform length, in tiles, for rail stations
+	uint8_t station_numtracks;                ///< the number of platforms to default on for rail stations
+	uint8_t station_platlength;               ///< the platform length, in tiles, for rail stations
 	bool   station_dragdrop;                 ///< whether drag and drop is enabled for stations
 	bool   station_show_coverage;            ///< whether to highlight coverage area
 	bool   persistent_buildingtools;         ///< keep the building tools active after usage
 	bool   expenses_layout;                  ///< layout of expenses window
 	uint32_t last_newgrf_count;                ///< the numbers of NewGRFs we found during the last scan
-	byte   missing_strings_threshold;        ///< the number of missing strings before showing the warning
+	uint8_t missing_strings_threshold;        ///< the number of missing strings before showing the warning
 	uint8_t  graph_line_thickness;             ///< the thickness of the lines in the various graph guis
 	uint8_t  osk_activation;                   ///< Mouse gesture to trigger the OSK.
-	Colours starting_colour;                 ///< default color scheme for the company to start a new game with
-	Colours starting_colour_secondary;       ///< default secondary color scheme for the company to start a new game with
+	Colours starting_colour;                 ///< default colour scheme for the company to start a new game with
+	Colours starting_colour_secondary;       ///< default secondary colour scheme for the company to start a new game with
 	bool   show_newgrf_name;                 ///< Show the name of the NewGRF in the build vehicle window
 	bool   show_cargo_in_vehicle_lists;      ///< Show the cargoes the vehicles can carry in the list windows
 	bool   auto_remove_signals;              ///< automatically remove signals when in the way during rail construction
@@ -246,25 +281,25 @@ struct SoundSettings {
 
 /** Settings related to music. */
 struct MusicSettings {
-	byte playlist;     ///< The playlist (number) to play
-	byte music_vol;    ///< The requested music volume
-	byte effect_vol;   ///< The requested effects volume
-	byte custom_1[33]; ///< The order of the first custom playlist
-	byte custom_2[33]; ///< The order of the second custom playlist
+	uint8_t playlist;     ///< The playlist (number) to play
+	uint8_t music_vol;    ///< The requested music volume
+	uint8_t effect_vol;   ///< The requested effects volume
+	uint8_t custom_1[33]; ///< The order of the first custom playlist
+	uint8_t custom_2[33]; ///< The order of the second custom playlist
 	bool playing;      ///< Whether music is playing
 	bool shuffle;      ///< Whether to shuffle the music
 };
 
 /** Settings related to currency/unit systems. */
 struct LocaleSettings {
-	byte        currency;                         ///< currency we currently use
-	byte        units_velocity;                   ///< unit system for velocity of trains and road vehicles
-	byte        units_velocity_nautical;          ///< unit system for velocity of ships and aircraft
-	byte        units_power;                      ///< unit system for power
-	byte        units_weight;                     ///< unit system for weight
-	byte        units_volume;                     ///< unit system for volume
-	byte        units_force;                      ///< unit system for force
-	byte        units_height;                     ///< unit system for height
+	uint8_t      currency;                        ///< currency we currently use
+	uint8_t      units_velocity;                  ///< unit system for velocity of trains and road vehicles
+	uint8_t      units_velocity_nautical;         ///< unit system for velocity of ships and aircraft
+	uint8_t      units_power;                     ///< unit system for power
+	uint8_t      units_weight;                    ///< unit system for weight
+	uint8_t      units_volume;                    ///< unit system for volume
+	uint8_t      units_force;                     ///< unit system for force
+	uint8_t      units_height;                    ///< unit system for height
 	std::string digit_group_separator;            ///< thousand separator for non-currencies
 	std::string digit_group_separator_currency;   ///< thousand separator for currencies
 	std::string digit_decimal_separator;          ///< decimal separator
@@ -313,15 +348,18 @@ struct NetworkSettings {
 	std::string server_invite_code_secret;                ///< Secret to proof we got this invite code from the Game Coordinator.
 	std::string server_name;                              ///< name of the server
 	std::string server_password;                          ///< password for joining this server
+	NetworkAuthorizedKeys server_authorized_keys; ///< Public keys of clients that are authorized to connect to the game.
 	std::string rcon_password;                            ///< password for rconsole (server side)
+	NetworkAuthorizedKeys rcon_authorized_keys; ///< Public keys of clients that are authorized to use the rconsole (server side).
+	bool allow_insecure_admin_login; ///< Whether to allow logging in as admin using the insecure old JOIN packet.
 	std::string admin_password;                           ///< password for the admin network
+	NetworkAuthorizedKeys admin_authorized_keys; ///< Public keys of clients that are authorized to use the admin network.
 	std::string client_name;                              ///< name of the player (as client)
-	std::string default_company_pass;                     ///< default password for new companies in encrypted form
+	std::string client_secret_key; ///< The secret key of the client for authorized key logins.
+	std::string client_public_key; ///< The public key of the client for authorized key logins.
 	std::string connect_to_ip;                            ///< default for the "Add server" query
-	std::string network_id;                               ///< network ID for servers
 	bool        autoclean_companies;                      ///< automatically remove companies that are not in use
-	uint8_t       autoclean_unprotected;                    ///< remove passwordless companies after this many months
-	uint8_t       autoclean_protected;                      ///< remove the password from passworded companies after this many months
+	uint8_t       autoclean_protected; ///< Remove companies after this many months.
 	uint8_t       autoclean_novehicles;                     ///< remove companies with no vehicles after this many months
 	uint8_t       max_companies;                            ///< maximum amount of companies
 	uint8_t       max_clients;                              ///< maximum amount of clients
@@ -332,6 +370,8 @@ struct NetworkSettings {
 	std::string last_joined;                              ///< Last joined server
 	UseRelayService use_relay_service;                    ///< Use relay service?
 	ParticipateSurvey participate_survey;                 ///< Participate in the automated survey
+
+	bool AdminAuthenticationConfigured() const { return !this->admin_password.empty() || !this->admin_authorized_keys.empty(); }
 };
 
 /** Settings related to the creation of games. */
@@ -341,27 +381,28 @@ struct GameCreationSettings {
 	TimerGameCalendar::Year ending_year;     ///< scoring end date
 	uint8_t  map_x;                            ///< X size of map
 	uint8_t  map_y;                            ///< Y size of map
-	byte   land_generator;                   ///< the landscape generator
-	byte   oil_refinery_limit;               ///< distance oil refineries allowed from map edge
-	byte   snow_line_height;                 ///< the configured snow line height (deduced from "snow_coverage")
-	byte   snow_coverage;                    ///< the amount of snow coverage on the map
-	byte   desert_coverage;                  ///< the amount of desert coverage on the map
-	byte   heightmap_height;                 ///< highest mountain for heightmap (towards what it scales)
-	byte   tgen_smoothness;                  ///< how rough is the terrain from 0-3
-	byte   tree_placer;                      ///< the tree placer algorithm
-	byte   heightmap_rotation;               ///< rotation director for the heightmap
-	byte   se_flat_world_height;             ///< land height a flat world gets in SE
-	byte   town_name;                        ///< the town name generator used for town names
-	byte   landscape;                        ///< the landscape we're currently in
-	byte   water_borders;                    ///< bitset of the borders that are water
+	uint8_t land_generator;                   ///< the landscape generator
+	uint8_t oil_refinery_limit;               ///< distance oil refineries allowed from map edge
+	uint8_t snow_line_height;                 ///< the configured snow line height (deduced from "snow_coverage")
+	uint8_t snow_coverage;                    ///< the amount of snow coverage on the map
+	uint8_t desert_coverage;                  ///< the amount of desert coverage on the map
+	uint8_t heightmap_height;                 ///< highest mountain for heightmap (towards what it scales)
+	uint8_t tgen_smoothness;                  ///< how rough is the terrain from 0-3
+	uint8_t tree_placer;                      ///< the tree placer algorithm
+	uint8_t heightmap_rotation;               ///< rotation director for the heightmap
+	uint8_t se_flat_world_height;             ///< land height a flat world gets in SE
+	uint8_t town_name;                        ///< the town name generator used for town names
+	LandscapeType landscape;                        ///< the landscape we're currently in
+	BorderFlags water_borders;                    ///< bitset of the borders that are water
+	BorderFlagPresets water_border_presets;    ///< presets for map border options
 	uint16_t custom_town_number;               ///< manually entered number of towns
 	uint16_t custom_industry_number;           ///< manually entered number of industries
-	byte   variety;                          ///< variety level applied to TGP
-	byte   custom_terrain_type;              ///< manually entered height for TGP to aim for
-	byte   custom_sea_level;                 ///< manually entered percentage of water in the map
-	byte   min_river_length;                 ///< the minimum river length
-	byte   river_route_random;               ///< the amount of randomicity for the route finding
-	byte   amount_of_rivers;                 ///< the amount of rivers
+	uint8_t variety;                          ///< variety level applied to TGP
+	uint8_t custom_terrain_type;              ///< manually entered height for TGP to aim for
+	uint8_t custom_sea_level;                 ///< manually entered percentage of water in the map
+	uint8_t min_river_length;                 ///< the minimum river length
+	uint8_t river_route_random;               ///< the amount of randomicity for the route finding
+	uint8_t amount_of_rivers;                 ///< the amount of rivers
 };
 
 /** Settings related to construction in-game */
@@ -370,9 +411,9 @@ struct ConstructionSettings {
 	bool   build_on_slopes;                  ///< allow building on slopes
 	bool   autoslope;                        ///< allow terraforming under things
 	uint16_t max_bridge_length;                ///< maximum length of bridges
-	byte   max_bridge_height;                ///< maximum height of bridges
+	uint8_t max_bridge_height;                ///< maximum height of bridges
 	uint16_t max_tunnel_length;                ///< maximum length of tunnels
-	byte   train_signal_side;                ///< show signals on left / driving / right side
+	uint8_t train_signal_side;                ///< show signals on left / driving / right side
 	bool   extra_dynamite;                   ///< extra dynamite
 	bool   road_stop_on_town_road;           ///< allow building of drive-through road stops on town owned roads
 	bool   road_stop_on_competitor_road;     ///< allow building of drive-through road stops on roads owned by competitors
@@ -381,7 +422,7 @@ struct ConstructionSettings {
 	uint8_t  industry_platform;                ///< the amount of flat land around an industry
 	bool   freeform_edges;                   ///< allow terraforming the tiles at the map edges
 	uint8_t  extra_tree_placement;             ///< (dis)allow building extra trees in-game
-	uint8_t  command_pause_level;              ///< level/amount of commands that can't be executed while paused
+	CommandPauseLevel command_pause_level; ///< level/amount of commands that can't be executed while paused
 
 	uint32_t terraform_per_64k_frames;         ///< how many tile heights may, over a long period, be terraformed per 65536 frames?
 	uint16_t terraform_frame_burst;            ///< how many tile heights may, over a short period, be terraformed?
@@ -408,42 +449,10 @@ struct ScriptSettings {
 	uint32_t script_max_memory_megabytes;      ///< limit on memory a single script instance may have allocated
 };
 
-/** Settings related to the new pathfinder. */
-struct NPFSettings {
-	/**
-	 * The maximum amount of search nodes a single NPF run should take. This
-	 * limit should make sure performance stays at acceptable levels at the cost
-	 * of not being perfect anymore.
-	 */
-	uint32_t npf_max_search_nodes;
-	uint32_t maximum_go_to_depot_penalty;      ///< What is the maximum penalty that may be endured for going to a depot
-
-	uint32_t npf_rail_firstred_penalty;        ///< the penalty for when the first signal is red (and it is not an exit or combo signal)
-	uint32_t npf_rail_firstred_exit_penalty;   ///< the penalty for when the first signal is red (and it is an exit or combo signal)
-	uint32_t npf_rail_lastred_penalty;         ///< the penalty for when the last signal is red
-	uint32_t npf_rail_station_penalty;         ///< the penalty for station tiles
-	uint32_t npf_rail_slope_penalty;           ///< the penalty for sloping upwards
-	uint32_t npf_rail_curve_penalty;           ///< the penalty for curves
-	uint32_t npf_rail_depot_reverse_penalty;   ///< the penalty for reversing in depots
-	uint32_t npf_rail_pbs_cross_penalty;       ///< the penalty for crossing a reserved rail track
-	uint32_t npf_rail_pbs_signal_back_penalty; ///< the penalty for passing a pbs signal from the backside
-	uint32_t npf_buoy_penalty;                 ///< the penalty for going over (through) a buoy
-	uint32_t npf_water_curve_penalty;          ///< the penalty for curves
-	uint32_t npf_road_curve_penalty;           ///< the penalty for curves
-	uint32_t npf_crossing_penalty;             ///< the penalty for level crossings
-	uint32_t npf_road_drive_through_penalty;   ///< the penalty for going through a drive-through road stop
-	uint32_t npf_road_dt_occupied_penalty;     ///< the penalty multiplied by the fill percentage of a drive-through road stop
-	uint32_t npf_road_bay_occupied_penalty;    ///< the penalty multiplied by the fill percentage of a road bay
-};
-
 /** Settings related to the yet another pathfinder. */
 struct YAPFSettings {
-	bool   disable_node_optimization;        ///< whether to use exit-dir instead of trackdir in node key
 	uint32_t max_search_nodes;                 ///< stop path-finding when this number of nodes visited
 	uint32_t maximum_go_to_depot_penalty;      ///< What is the maximum penalty that may be endured for going to a depot
-	bool   ship_use_yapf;                    ///< use YAPF for ships
-	bool   road_use_yapf;                    ///< use YAPF for road
-	bool   rail_use_yapf;                    ///< use YAPF for rail
 	uint32_t road_slope_penalty;               ///< penalty for up-hill slope
 	uint32_t road_curve_penalty;               ///< penalty for curves
 	uint32_t road_crossing_penalty;            ///< penalty for level crossing
@@ -480,23 +489,17 @@ struct YAPFSettings {
 
 /** Settings related to all pathfinders. */
 struct PathfinderSettings {
-	uint8_t  pathfinder_for_trains;            ///< the pathfinder to use for trains
-	uint8_t  pathfinder_for_roadvehs;          ///< the pathfinder to use for roadvehicles
-	uint8_t  pathfinder_for_ships;             ///< the pathfinder to use for ships
-	bool   new_pathfinding_all;              ///< use the newest pathfinding algorithm for all
-
 	bool   roadveh_queue;                    ///< buggy road vehicle queueing
 	bool   forbid_90_deg;                    ///< forbid trains to make 90 deg turns
 
 	bool   reverse_at_signals;               ///< whether to reverse at signals at all
-	byte   wait_oneway_signal;               ///< waitingtime in days before a oneway signal
-	byte   wait_twoway_signal;               ///< waitingtime in days before a twoway signal
+	uint8_t wait_oneway_signal;               ///< waitingtime in days before a oneway signal
+	uint8_t wait_twoway_signal;               ///< waitingtime in days before a twoway signal
 
 	bool   reserve_paths;                    ///< always reserve paths regardless of signal type.
-	byte   wait_for_pbs_path;                ///< how long to wait for a path reservation.
-	byte   path_backoff_interval;            ///< ticks between checks for a free path.
+	uint8_t wait_for_pbs_path;                ///< how long to wait for a path reservation.
+	uint8_t path_backoff_interval;            ///< ticks between checks for a free path.
 
-	NPFSettings  npf;                        ///< pathfinder settings for the new pathfinder
 	YAPFSettings yapf;                       ///< pathfinder settings for the yet another pathfinder
 };
 
@@ -528,8 +531,8 @@ struct VehicleSettings {
 	uint8_t  freight_trains;                   ///< value to multiply the weight of cargo by
 	bool   dynamic_engines;                  ///< enable dynamic allocation of engine data
 	bool   never_expire_vehicles;            ///< never expire vehicles
-	byte   extend_vehicle_life;              ///< extend vehicle life by this many years
-	byte   road_side;                        ///< the side of the road vehicles drive on
+	uint8_t extend_vehicle_life;              ///< extend vehicle life by this many years
+	uint8_t road_side;                        ///< the side of the road vehicles drive on
 	uint8_t  plane_crashes;                    ///< number of plane crashes, 0 = none, 1 = reduced, 2 = normal
 };
 
@@ -539,7 +542,7 @@ struct EconomySettings {
 	bool   bribe;                            ///< enable bribing the local authority
 	EconomyType type;                        ///< economy type (original/smooth/frozen)
 	uint8_t  feeder_payment_share;             ///< percentage of leg payment to virtually pay in feeder systems
-	byte   dist_local_authority;             ///< distance for town local authority, default 20
+	uint8_t dist_local_authority;             ///< distance for town local authority, default 20
 	bool   exclusive_rights;                 ///< allow buying exclusive rights
 	bool   fund_buildings;                   ///< allow funding new buildings
 	bool   fund_roads;                       ///< allow funding local road reconstruction
@@ -553,6 +556,7 @@ struct EconomySettings {
 	TownCargoGenMode town_cargogen_mode;     ///< algorithm for generating cargo from houses, @see TownCargoGenMode
 	bool   allow_town_roads;                 ///< towns are allowed to build roads (always allowed when generating world / in SE)
 	TownFounding found_town;                 ///< town founding.
+	PlaceHouses place_houses;                ///< players are allowed to place town houses.
 	bool   station_noise_level;              ///< build new airports when the town noise level is still within accepted limits
 	uint16_t town_noise_population[4];         ///< population to base decision on noise evaluation (@see town_council_tolerance)
 	bool   allow_town_level_crossings;       ///< towns are allowed to build level crossings
@@ -575,11 +579,11 @@ struct LinkGraphSettings {
 	uint8_t demand_distance;                  ///< influence of distance between stations on the demand function
 	uint8_t short_path_saturation;            ///< percentage up to which short paths are saturated before saturating most capacious paths
 
-	inline DistributionType GetDistributionType(CargoID cargo) const
+	inline DistributionType GetDistributionType(CargoType cargo) const
 	{
-		if (IsCargoInClass(cargo, CC_PASSENGERS)) return this->distribution_pax;
-		if (IsCargoInClass(cargo, CC_MAIL)) return this->distribution_mail;
-		if (IsCargoInClass(cargo, CC_ARMOURED)) return this->distribution_armoured;
+		if (IsCargoInClass(cargo, CargoClass::Passengers)) return this->distribution_pax;
+		if (IsCargoInClass(cargo, CargoClass::Mail)) return this->distribution_mail;
+		if (IsCargoInClass(cargo, CargoClass::Armoured)) return this->distribution_armoured;
 		return this->distribution_default;
 	}
 };
@@ -588,28 +592,40 @@ struct LinkGraphSettings {
 struct StationSettings {
 	bool   modified_catchment;               ///< different-size catchment areas
 	bool   serve_neutral_industries;         ///< company stations can serve industries with attached neutral stations
-	bool   adjacent_stations;                ///< allow stations to be built directly adjacent to other stations
 	bool   distant_join_stations;            ///< allow to join non-adjacent stations
 	bool   never_expire_airports;            ///< never expire airports
-	byte   station_spread;                   ///< amount a station may spread
+	uint8_t station_spread;                  ///< amount a station may spread
 };
 
 /** Default settings for vehicles. */
 struct VehicleDefaultSettings {
-	bool   servint_ispercent;                ///< service intervals are in percents
-	uint16_t servint_trains;                   ///< service interval for trains
-	uint16_t servint_roadveh;                  ///< service interval for road vehicles
-	uint16_t servint_aircraft;                 ///< service interval for aircraft
-	uint16_t servint_ships;                    ///< service interval for ships
+	bool   servint_ispercent = false; ///< service intervals are in percents
+	uint16_t servint_trains = 0; ///< service interval for trains
+	uint16_t servint_roadveh = 0; ///< service interval for road vehicles
+	uint16_t servint_aircraft = 0; ///< service interval for aircraft
+	uint16_t servint_ships = 0; ///< service interval for ships
 };
 
 /** Settings that can be set per company. */
 struct CompanySettings {
-	bool engine_renew;                       ///< is autorenew enabled
-	int16_t engine_renew_months;               ///< months before/after the maximum vehicle age a vehicle should be renewed
-	uint32_t engine_renew_money;               ///< minimum amount of money before autorenew is used
-	bool renew_keep_length;                  ///< sell some wagons if after autoreplace the train is longer than before
-	VehicleDefaultSettings vehicle;          ///< default settings for vehicles
+	bool engine_renew = false; ///< is autorenew enabled
+	int16_t engine_renew_months = 0; ///< months before/after the maximum vehicle age a vehicle should be renewed
+	uint32_t engine_renew_money = 0; ///< minimum amount of money before autorenew is used
+	bool renew_keep_length = false; ///< sell some wagons if after autoreplace the train is longer than before
+	VehicleDefaultSettings vehicle{}; ///< default settings for vehicles
+};
+
+/** Container for AI and Game script configuration. */
+struct ScriptConfigSettings
+{
+	TypedIndexContainer<std::array<std::unique_ptr<class AIConfig>, MAX_COMPANIES>, CompanyID> ai; ///< settings per company
+	std::unique_ptr<class GameConfig> game; ///< settings for gamescript
+
+	ScriptConfigSettings();
+	~ScriptConfigSettings();
+
+	ScriptConfigSettings(const ScriptConfigSettings &other);
+	ScriptConfigSettings &operator=(const ScriptConfigSettings &other);
 };
 
 /** All settings together for the game. */
@@ -619,8 +635,7 @@ struct GameSettings {
 	ConstructionSettings construction;       ///< construction of things in-game
 	AISettings           ai;                 ///< what may the AI do?
 	ScriptSettings       script;             ///< settings for scripts
-	class AIConfig      *ai_config[MAX_COMPANIES]; ///< settings per company
-	class GameConfig    *game_config;        ///< settings for gamescript
+	ScriptConfigSettings script_config;      ///< AI and Gamescript configuration.
 	PathfinderSettings   pf;                 ///< settings for all pathfinders
 	OrderSettings        order;              ///< settings related to orders
 	VehicleSettings      vehicle;            ///< options for vehicles
