@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file subsidy_gui.cpp GUI for subsidies. */
@@ -26,10 +26,10 @@
 #include "safeguards.h"
 
 struct SubsidyListWindow : Window {
-	Scrollbar *vscroll;
-	Dimension cargo_icon_size;
+	Scrollbar *vscroll = nullptr;
+	Dimension cargo_icon_size{};
 
-	SubsidyListWindow(WindowDesc *desc, WindowNumber window_number) : Window(desc)
+	SubsidyListWindow(WindowDesc &desc, WindowNumber window_number) : Window(desc)
 	{
 		this->CreateNestedTree();
 		this->vscroll = this->GetScrollbar(WID_SUL_SCROLLBAR);
@@ -82,9 +82,9 @@ struct SubsidyListWindow : Window {
 	{
 		/* determine src coordinate for subsidy and try to scroll to it */
 		TileIndex xy;
-		switch (s->src_type) {
-			case SourceType::Industry: xy = Industry::Get(s->src)->location.tile; break;
-			case SourceType::Town:     xy =     Town::Get(s->src)->xy; break;
+		switch (s->src.type) {
+			case SourceType::Industry: xy = Industry::Get(s->src.ToIndustryID())->location.tile; break;
+			case SourceType::Town:     xy =     Town::Get(s->src.ToTownID())->xy; break;
 			default: NOT_REACHED();
 		}
 
@@ -92,9 +92,9 @@ struct SubsidyListWindow : Window {
 			if (_ctrl_pressed) ShowExtraViewportWindow(xy);
 
 			/* otherwise determine dst coordinate for subsidy and scroll to it */
-			switch (s->dst_type) {
-				case SourceType::Industry: xy = Industry::Get(s->dst)->location.tile; break;
-				case SourceType::Town:     xy =     Town::Get(s->dst)->xy; break;
+			switch (s->dst.type) {
+				case SourceType::Industry: xy = Industry::Get(s->dst.ToIndustryID())->location.tile; break;
+				case SourceType::Town:     xy =     Town::Get(s->dst.ToTownID())->xy; break;
 				default: NOT_REACHED();
 			}
 
@@ -131,26 +131,26 @@ struct SubsidyListWindow : Window {
 		return 3 + num_awarded + num_not_awarded;
 	}
 
-	void UpdateWidgetSize(WidgetID widget, Dimension *size, [[maybe_unused]] const Dimension &padding, [[maybe_unused]] Dimension *fill, [[maybe_unused]] Dimension *resize) override
+	void UpdateWidgetSize(WidgetID widget, Dimension &size, [[maybe_unused]] const Dimension &padding, [[maybe_unused]] Dimension &fill, [[maybe_unused]] Dimension &resize) override
 	{
 		if (widget != WID_SUL_PANEL) return;
 		Dimension d = maxdim(GetStringBoundingBox(STR_SUBSIDIES_OFFERED_TITLE), GetStringBoundingBox(STR_SUBSIDIES_SUBSIDISED_TITLE));
 
-		resize->height = GetCharacterHeight(FS_NORMAL);
+		fill.height = resize.height = GetCharacterHeight(FS_NORMAL);
 
 		d.height *= 5;
 		d.width += WidgetDimensions::scaled.framerect.Horizontal();
 		d.height += WidgetDimensions::scaled.framerect.Vertical();
-		*size = maxdim(*size, d);
+		size = maxdim(size, d);
 	}
 
-	void DrawCargoIcon(const Rect &r, int y_offset, CargoID cid) const
+	void DrawCargoIcon(const Rect &r, int y_offset, CargoType cargo_type) const
 	{
 		bool rtl = _current_text_dir == TD_RTL;
-		SpriteID icon = CargoSpec::Get(cid)->GetCargoIcon();
+		SpriteID icon = CargoSpec::Get(cargo_type)->GetCargoIcon();
 		Dimension d = GetSpriteSize(icon);
 		Rect ir = r.WithWidth(this->cargo_icon_size.width, rtl).WithHeight(GetCharacterHeight(FS_NORMAL));
-		DrawSprite(icon, PAL_NONE, CenterBounds(ir.left, ir.right, d.width), CenterBounds(ir.top, ir.bottom, this->cargo_icon_size.height) + y_offset);
+		DrawSprite(icon, PAL_NONE, CentreBounds(ir.left, ir.right, d.width), CentreBounds(ir.top, ir.bottom, this->cargo_icon_size.height) + y_offset);
 	}
 
 	void DrawWidget(const Rect &r, WidgetID widget) const override
@@ -174,18 +174,24 @@ struct SubsidyListWindow : Window {
 			if (!s->IsAwarded()) {
 				if (IsInsideMM(pos, 0, cap)) {
 					/* Displays the two offered towns */
-					SetupSubsidyDecodeParam(s, SubsidyDecodeParamType::Gui);
+					const CargoSpec *cs = CargoSpec::Get(s->cargo_type);
+					std::string text;
+
 					/* If using wallclock units, show minutes remaining. Otherwise show the date when the subsidy ends. */
 					if (TimerGameEconomy::UsingWallclockUnits()) {
-						SetDParam(7, STR_SUBSIDIES_OFFERED_EXPIRY_TIME);
-						SetDParam(8, s->remaining + 1); // We get the rest of the current economy month for free, since the expiration is checked on each new month.
+						text = GetString(STR_SUBSIDIES_OFFERED_FROM_TO,
+							cs->name, s->src.GetFormat(), s->src.id, s->dst.GetFormat(), s->dst.id,
+							STR_SUBSIDIES_OFFERED_EXPIRY_TIME,
+							s->remaining + 1); // We get the rest of the current economy month for free, since the expiration is checked on each new month.
 					} else {
-						SetDParam(7, STR_SUBSIDIES_OFFERED_EXPIRY_DATE);
-						SetDParam(8, TimerGameEconomy::date - ymd.day + s->remaining * 32);
+						text = GetString(STR_SUBSIDIES_OFFERED_FROM_TO,
+							cs->name, s->src.GetFormat(), s->src.id, s->dst.GetFormat(), s->dst.id,
+							STR_SUBSIDIES_OFFERED_EXPIRY_DATE,
+							TimerGameEconomy::date.base() - ymd.day + s->remaining * 32);
 					}
 
 					DrawCargoIcon(tr, pos * GetCharacterHeight(FS_NORMAL), s->cargo_type);
-					DrawString(sr.left, sr.right, sr.top + pos * GetCharacterHeight(FS_NORMAL), STR_SUBSIDIES_OFFERED_FROM_TO);
+					DrawString(sr.left, sr.right, sr.top + pos * GetCharacterHeight(FS_NORMAL), text);
 				}
 				pos++;
 				num++;
@@ -206,21 +212,27 @@ struct SubsidyListWindow : Window {
 		for (const Subsidy *s : Subsidy::Iterate()) {
 			if (s->IsAwarded()) {
 				if (IsInsideMM(pos, 0, cap)) {
-					SetupSubsidyDecodeParam(s, SubsidyDecodeParamType::Gui);
-					SetDParam(7, s->awarded);
+					const CargoSpec *cs = CargoSpec::Get(s->cargo_type);
+					std::string text;
+
 					/* If using wallclock units, show minutes remaining. Otherwise show the date when the subsidy ends. */
 					if (TimerGameEconomy::UsingWallclockUnits()) {
-						SetDParam(8, STR_SUBSIDIES_SUBSIDISED_EXPIRY_TIME);
-						SetDParam(9, s->remaining);
-					}
-					else {
-						SetDParam(8, STR_SUBSIDIES_SUBSIDISED_EXPIRY_DATE);
-						SetDParam(9, TimerGameEconomy::date - ymd.day + s->remaining * 32);
+						text = GetString(STR_SUBSIDIES_SUBSIDISED_FROM_TO,
+							cs->name, s->src.GetFormat(), s->src.id, s->dst.GetFormat(), s->dst.id,
+							s->awarded,
+							STR_SUBSIDIES_SUBSIDISED_EXPIRY_TIME,
+							s->remaining + 1); // We get the rest of the current economy month for free, since the expiration is checked on each new month.
+					} else {
+						text = GetString(STR_SUBSIDIES_SUBSIDISED_FROM_TO,
+							cs->name, s->src.GetFormat(), s->src.id, s->dst.GetFormat(), s->dst.id,
+							s->awarded,
+							STR_SUBSIDIES_SUBSIDISED_EXPIRY_DATE,
+							TimerGameEconomy::date.base() - ymd.day + s->remaining * 32);
 					}
 
 					/* Displays the two connected stations */
 					DrawCargoIcon(tr, pos * GetCharacterHeight(FS_NORMAL), s->cargo_type);
-					DrawString(sr.left, sr.right, sr.top + pos * GetCharacterHeight(FS_NORMAL), STR_SUBSIDIES_SUBSIDISED_FROM_TO);
+					DrawString(sr.left, sr.right, sr.top + pos * GetCharacterHeight(FS_NORMAL), text);
 				}
 				pos++;
 				num++;
@@ -235,7 +247,7 @@ struct SubsidyListWindow : Window {
 
 	void OnResize() override
 	{
-		this->vscroll->SetCapacityFromWidget(this, WID_SUL_PANEL);
+		this->vscroll->SetCapacityFromWidget(this, WID_SUL_PANEL, WidgetDimensions::scaled.framerect.Vertical());
 	}
 
 	/**
@@ -250,16 +262,16 @@ struct SubsidyListWindow : Window {
 	}
 };
 
-static constexpr NWidgetPart _nested_subsidies_list_widgets[] = {
+static constexpr std::initializer_list<NWidgetPart> _nested_subsidies_list_widgets = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_BROWN),
-		NWidget(WWT_CAPTION, COLOUR_BROWN), SetDataTip(STR_SUBSIDIES_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_CAPTION, COLOUR_BROWN), SetStringTip(STR_SUBSIDIES_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 		NWidget(WWT_SHADEBOX, COLOUR_BROWN),
 		NWidget(WWT_DEFSIZEBOX, COLOUR_BROWN),
 		NWidget(WWT_STICKYBOX, COLOUR_BROWN),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_BROWN, WID_SUL_PANEL), SetDataTip(0x0, STR_SUBSIDIES_TOOLTIP_CLICK_ON_SERVICE_TO_CENTER), SetResize(1, 1), SetScrollbar(WID_SUL_SCROLLBAR), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_BROWN, WID_SUL_PANEL), SetToolTip(STR_SUBSIDIES_TOOLTIP_CLICK_ON_SERVICE_TO_CENTER), SetResize(1, 1), SetScrollbar(WID_SUL_SCROLLBAR), EndContainer(),
 		NWidget(NWID_VERTICAL),
 			NWidget(NWID_VSCROLLBAR, COLOUR_BROWN, WID_SUL_SCROLLBAR),
 			NWidget(WWT_RESIZEBOX, COLOUR_BROWN),
@@ -267,15 +279,15 @@ static constexpr NWidgetPart _nested_subsidies_list_widgets[] = {
 	EndContainer(),
 };
 
-static WindowDesc _subsidies_list_desc(__FILE__, __LINE__,
+static WindowDesc _subsidies_list_desc(
 	WDP_AUTO, "list_subsidies", 500, 127,
 	WC_SUBSIDIES_LIST, WC_NONE,
-	0,
-	std::begin(_nested_subsidies_list_widgets), std::end(_nested_subsidies_list_widgets)
+	{},
+	_nested_subsidies_list_widgets
 );
 
 
 void ShowSubsidiesList()
 {
-	AllocateWindowDescFront<SubsidyListWindow>(&_subsidies_list_desc, 0);
+	AllocateWindowDescFront<SubsidyListWindow>(_subsidies_list_desc, 0);
 }
