@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /**
@@ -10,7 +10,7 @@
  */
 
 /** Height map colours for the green colour scheme, ordered by height. */
-static const uint32 _green_map_heights[] = {
+static const uint32_t _green_map_heights[] = {
 	MKCOLOUR(0x59595958),
 	MKCOLOUR(0x59595958),
 	MKCOLOUR(0x59595959),
@@ -129,7 +129,7 @@ static const uint32 _green_map_heights[] = {
 };
 
 /** Height map colours for the dark green colour scheme, ordered by height. */
-static const uint32 _dark_green_map_heights[] = {
+static const uint32_t _dark_green_map_heights[] = {
 	MKCOLOUR(0x60606060),
 	MKCOLOUR(0x60606061),
 	MKCOLOUR(0x60606160),
@@ -232,7 +232,7 @@ static const uint32 _dark_green_map_heights[] = {
 };
 
 /** Height map colours for the violet colour scheme, ordered by height. */
-static const uint32 _violet_map_heights[] = {
+static const uint32_t _violet_map_heights[] = {
 	MKCOLOUR(0x80808080),
 	MKCOLOUR(0x80808081),
 	MKCOLOUR(0x80808180),
