@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file timetable_gui.cpp GUI for time tabling. */
@@ -27,6 +27,7 @@
 #include "settings_type.h"
 #include "timetable_cmd.h"
 #include "timetable.h"
+#include "core/string_consumer.hpp"
 
 #include "widgets/timetable_widget.h"
 
@@ -42,26 +43,16 @@ struct TimetableArrivalDeparture {
 };
 
 /**
- * Set the timetable parameters in the format as described by the setting.
- * @param param1 the first DParam to fill
- * @param param2 the second DParam to fill
- * @param ticks  the number of ticks to 'draw'
+ * Get parameters to format timetable time.
+ * @param ticks Number of ticks to format.
+ * @returns Pair of parameters to format timetable time.
  */
-void SetTimetableParams(int param1, int param2, TimerGameTick::Ticks ticks)
+std::pair<StringParameter, StringParameter> GetTimetableParameters(TimerGameTick::Ticks ticks)
 {
 	switch (_settings_client.gui.timetable_mode) {
-		case TimetableMode::Days:
-			SetDParam(param1, STR_UNITS_DAYS);
-			SetDParam(param2, ticks / Ticks::DAY_TICKS);
-			break;
-		case TimetableMode::Seconds:
-			SetDParam(param1, STR_UNITS_SECONDS);
-			SetDParam(param2, ticks / Ticks::TICKS_PER_SECOND);
-			break;
-		case TimetableMode::Ticks:
-			SetDParam(param1, STR_UNITS_TICKS);
-			SetDParam(param2, ticks);
-			break;
+		case TimetableMode::Days: return {STR_UNITS_DAYS, ticks / Ticks::DAY_TICKS};
+		case TimetableMode::Seconds: return {STR_UNITS_SECONDS, ticks / Ticks::TICKS_PER_SECOND};
+		case TimetableMode::Ticks: return {STR_UNITS_TICKS, ticks};
 		default:
 			NOT_REACHED();
 	}
@@ -86,20 +77,20 @@ static inline TimerGameTick::Ticks TicksPerTimetableUnit()
 }
 
 /**
- * Determine if a vehicle should be shown as late, depending on the timetable display setting.
- * @param v The vehicle in question.
+ * Determine if a vehicle should be shown as late or early, using a threshold depending on the timetable display setting.
+ * @param ticks The number of ticks that the vehicle is late or early.
  * @param round_to_day When using ticks, if we should round up to the nearest day.
- * @return True if the vehicle is later than the threshold.
+ * @return True if the vehicle is outside the "on time" threshold, either early or late.
  */
-bool VehicleIsAboveLatenessThreshold(const Vehicle *v, bool round_to_day)
+bool VehicleIsAboveLatenessThreshold(TimerGameTick::Ticks ticks, bool round_to_day)
 {
 	switch (_settings_client.gui.timetable_mode) {
 		case TimetableMode::Days:
-			return v->lateness_counter > Ticks::DAY_TICKS;
+			return ticks > Ticks::DAY_TICKS;
 		case TimetableMode::Seconds:
-			return v->lateness_counter > Ticks::TICKS_PER_SECOND;
+			return ticks > Ticks::TICKS_PER_SECOND;
 		case TimetableMode::Ticks:
-			return v->lateness_counter > (round_to_day ? Ticks::DAY_TICKS : 0);
+			return ticks > (round_to_day ? Ticks::DAY_TICKS : 0);
 		default:
 			NOT_REACHED();
 	}
@@ -111,15 +102,15 @@ bool VehicleIsAboveLatenessThreshold(const Vehicle *v, bool round_to_day)
  * @param travelling whether we are interested in the travel or the wait part.
  * @return true if the travel/wait time can be used.
  */
-static bool CanDetermineTimeTaken(const Order *order, bool travelling)
+static bool CanDetermineTimeTaken(const Order &order, bool travelling)
 {
 	/* Current order is conditional */
-	if (order->IsType(OT_CONDITIONAL) || order->IsType(OT_IMPLICIT)) return false;
+	if (order.IsType(OT_CONDITIONAL) || order.IsType(OT_IMPLICIT)) return false;
 	/* No travel time and we have not already finished travelling */
-	if (travelling && !order->IsTravelTimetabled()) return false;
+	if (travelling && !order.IsTravelTimetabled()) return false;
 	/* No wait time but we are loading at this timetabled station */
-	if (!travelling && !order->IsWaitTimetabled() && order->IsType(OT_GOTO_STATION) &&
-			!(order->GetNonStopType() & ONSF_NO_STOP_AT_DESTINATION_STATION)) {
+	if (!travelling && !order.IsWaitTimetabled() && order.IsType(OT_GOTO_STATION) &&
+			!order.GetNonStopType().Test(OrderNonStopFlag::NoDestination)) {
 		return false;
 	}
 
@@ -148,7 +139,7 @@ static void FillTimetableArrivalDepartureTable(const Vehicle *v, VehicleOrderID
 
 	TimerGameTick::Ticks sum = offset;
 	VehicleOrderID i = start;
-	const Order *order = v->GetOrder(i);
+	auto orders = v->Orders();
 
 	/* Cyclically loop over all orders until we reach the current one again.
 	 * As we may start at the current order, do a post-checking loop */
@@ -156,62 +147,42 @@ static void FillTimetableArrivalDepartureTable(const Vehicle *v, VehicleOrderID
 		/* Automatic orders don't influence the overall timetable;
 		 * they just add some untimetabled entries, but the time till
 		 * the next non-implicit order can still be known. */
-		if (!order->IsType(OT_IMPLICIT)) {
+		if (!orders[i].IsType(OT_IMPLICIT)) {
 			if (travelling || i != start) {
-				if (!CanDetermineTimeTaken(order, true)) return;
-				sum += order->GetTimetabledTravel();
+				if (!CanDetermineTimeTaken(orders[i], true)) return;
+				sum += orders[i].GetTimetabledTravel();
 				table[i].arrival = sum;
 			}
 
-			if (!CanDetermineTimeTaken(order, false)) return;
-			sum += order->GetTimetabledWait();
+			if (!CanDetermineTimeTaken(orders[i], false)) return;
+			sum += orders[i].GetTimetabledWait();
 			table[i].departure = sum;
 		}
 
-		++i;
-		order = order->next;
-		if (i >= v->GetNumOrders()) {
-			i = 0;
-			assert(order == nullptr);
-			order = v->orders->GetFirstOrder();
-		}
+		i = v->orders->GetNext(i);
 	} while (i != start);
 
 	/* When loading at a scheduled station we still have to treat the
 	 * travelling part of the first order. */
 	if (!travelling) {
-		if (!CanDetermineTimeTaken(order, true)) return;
-		sum += order->GetTimetabledTravel();
+		if (!CanDetermineTimeTaken(orders[i], true)) return;
+		sum += orders[i].GetTimetabledTravel();
 		table[i].arrival = sum;
 	}
 }
 
-
-/**
- * Callback for when a time has been chosen to start the time table
- * @param w the window related to the setting of the date
- * @param date the actually chosen date
- */
-static void ChangeTimetableStartCallback(const Window *w, TimerGameEconomy::Date date, void *data)
-{
-	Command<CMD_SET_TIMETABLE_START>::Post(STR_ERROR_CAN_T_TIMETABLE_VEHICLE, (VehicleID)w->window_number, reinterpret_cast<std::uintptr_t>(data) != 0, GetStartTickFromDate(date));
-}
-
-
 struct TimetableWindow : Window {
-	int sel_index;
-	VehicleTimetableWidgets query_widget; ///< Which button was clicked to open the query text input?
-	const Vehicle *vehicle;    ///< Vehicle monitored by the window.
-	bool show_expected;        ///< Whether we show expected arrival or scheduled.
-	Scrollbar *vscroll;        ///< The scrollbar.
-	bool set_start_date_all;   ///< Set start date using minutes text entry for all timetable entries (ctrl-click) action.
-	bool change_timetable_all; ///< Set wait time or speed for all timetable entries (ctrl-click) action.
+	int sel_index = -1;
+	VehicleTimetableWidgets query_widget{}; ///< Which button was clicked to open the query text input?
+	const Vehicle *vehicle = nullptr; ///< Vehicle monitored by the window.
+	bool show_expected = true; ///< Whether we show expected arrival or scheduled.
+	Scrollbar *vscroll = nullptr; ///< The scrollbar.
+	bool set_start_date_all = false; ///< Set start date using minutes text entry for all timetable entries (ctrl-click) action.
+	bool change_timetable_all = false; ///< Set wait time or speed for all timetable entries (ctrl-click) action.
 
-	TimetableWindow(WindowDesc *desc, WindowNumber window_number) :
+	TimetableWindow(WindowDesc &desc, WindowNumber window_number) :
 			Window(desc),
-			sel_index(-1),
-			vehicle(Vehicle::Get(window_number)),
-			show_expected(true)
+			vehicle(Vehicle::Get(window_number))
 	{
 		this->CreateNestedTree();
 		this->vscroll = this->GetScrollbar(WID_VT_SCROLLBAR);
@@ -235,9 +206,9 @@ struct TimetableWindow : Window {
 	 */
 	static bool BuildArrivalDepartureList(const Vehicle *v, std::vector<TimetableArrivalDeparture> &table)
 	{
-		assert(HasBit(v->vehicle_flags, VF_TIMETABLE_STARTED));
+		assert(v->vehicle_flags.Test(VehicleFlag::TimetableStarted));
 
-		bool travelling = (!v->current_order.IsType(OT_LOADING) || v->current_order.GetNonStopType() == ONSF_STOP_EVERYWHERE);
+		bool travelling = (!v->current_order.IsType(OT_LOADING) || v->current_order.GetNonStopType().None());
 		TimerGameTick::Ticks start_time = -v->current_order_time;
 
 		/* If arrival and departure times are in days, compensate for the current date_fract. */
@@ -248,37 +219,42 @@ struct TimetableWindow : Window {
 		return (travelling && v->lateness_counter < 0);
 	}
 
-	void UpdateWidgetSize(WidgetID widget, Dimension *size, [[maybe_unused]] const Dimension &padding, [[maybe_unused]] Dimension *fill, [[maybe_unused]] Dimension *resize) override
+	void UpdateWidgetSize(WidgetID widget, Dimension &size, [[maybe_unused]] const Dimension &padding, [[maybe_unused]] Dimension &fill, [[maybe_unused]] Dimension &resize) override
 	{
 		switch (widget) {
 			case WID_VT_ARRIVAL_DEPARTURE_PANEL:
 				/* We handle this differently depending on the timetable mode. */
 				if (_settings_client.gui.timetable_mode == TimetableMode::Seconds) {
 					/* A five-digit number would fit a timetable lasting 2.7 real-world hours, which should be plenty. */
-					SetDParamMaxDigits(1, 4, FS_SMALL);
-					size->width = std::max(GetStringBoundingBox(STR_TIMETABLE_ARRIVAL_SECONDS_IN_FUTURE).width, GetStringBoundingBox(STR_TIMETABLE_DEPARTURE_SECONDS_IN_FUTURE).width) + WidgetDimensions::scaled.hsep_wide + padding.width;
+					uint64_t max_digits = GetParamMaxDigits(4, FS_SMALL);
+					size.width = std::max(
+							GetStringBoundingBox(GetString(STR_TIMETABLE_ARRIVAL_SECONDS_IN_FUTURE, TC_BLACK, max_digits)).width,
+							GetStringBoundingBox(GetString(STR_TIMETABLE_DEPARTURE_SECONDS_IN_FUTURE, TC_BLACK, max_digits)).width)
+							+ WidgetDimensions::scaled.hsep_wide + padding.width;
 				} else {
-					SetDParamMaxValue(1, TimerGameEconomy::DateAtStartOfYear(EconomyTime::MAX_YEAR), 0, FS_SMALL);
-					size->width = std::max(GetStringBoundingBox(STR_TIMETABLE_ARRIVAL_DATE).width, GetStringBoundingBox(STR_TIMETABLE_DEPARTURE_DATE).width) + WidgetDimensions::scaled.hsep_wide + padding.width;
+					uint64_t max_value = GetParamMaxValue(TimerGameEconomy::DateAtStartOfYear(EconomyTime::MAX_YEAR).base(), 0, FS_SMALL);
+					size.width = std::max(
+							GetStringBoundingBox(GetString(STR_TIMETABLE_ARRIVAL_DATE, TC_BLACK, max_value)).width,
+							GetStringBoundingBox(GetString(STR_TIMETABLE_DEPARTURE_DATE, TC_BLACK, max_value)).width)
+							+ WidgetDimensions::scaled.hsep_wide + padding.width;
 				}
 				[[fallthrough]];
 
-			case WID_VT_ARRIVAL_DEPARTURE_SELECTION:
 			case WID_VT_TIMETABLE_PANEL:
-				resize->height = GetCharacterHeight(FS_NORMAL);
-				size->height = 8 * resize->height + padding.height;
+				fill.height = resize.height = GetCharacterHeight(FS_NORMAL);
+				size.height = 8 * resize.height + padding.height;
 				break;
 
 			case WID_VT_SUMMARY_PANEL:
-				size->height = 2 * GetCharacterHeight(FS_NORMAL) + padding.height;
+				size.height = 2 * GetCharacterHeight(FS_NORMAL) + padding.height;
 				break;
 		}
 	}
 
 	int GetOrderFromTimetableWndPt(int y, [[maybe_unused]] const Vehicle *v)
 	{
-		int sel = this->vscroll->GetScrolledRowFromWidget(y, this, WID_VT_TIMETABLE_PANEL, WidgetDimensions::scaled.framerect.top);
-		if (sel == INT_MAX) return INVALID_ORDER;
+		int32_t sel = this->vscroll->GetScrolledRowFromWidget(y, this, WID_VT_TIMETABLE_PANEL, WidgetDimensions::scaled.framerect.top);
+		if (sel == INT32_MAX) return sel;
 		assert(IsInsideBS(sel, 0, v->GetNumOrders() * 2));
 		return sel;
 	}
@@ -372,7 +348,7 @@ struct TimetableWindow : Window {
 				if (selected % 2 != 0) {
 					disable = order != nullptr && (order->IsType(OT_CONDITIONAL) || order->IsType(OT_IMPLICIT));
 				} else {
-					disable = order == nullptr || ((!order->IsType(OT_GOTO_STATION) || (order->GetNonStopType() & ONSF_NO_STOP_AT_DESTINATION_STATION)) && !order->IsType(OT_CONDITIONAL));
+					disable = order == nullptr || ((!order->IsType(OT_GOTO_STATION) || order->GetNonStopType().Test(OrderNonStopFlag::NoDestination)) && !order->IsType(OT_CONDITIONAL));
 				}
 			}
 			bool disable_speed = disable || selected % 2 == 0 || v->type == VEH_AIRCRAFT;
@@ -397,19 +373,52 @@ struct TimetableWindow : Window {
 			this->DisableWidget(WID_VT_SHARED_ORDER_LIST);
 		}
 
-		this->SetWidgetLoweredState(WID_VT_AUTOFILL, HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE));
+		this->SetWidgetLoweredState(WID_VT_AUTOFILL, v->vehicle_flags.Test(VehicleFlag::AutofillTimetable));
 
 		this->DrawWidgets();
 	}
 
-	void SetStringParameters(WidgetID widget) const override
+	std::string GetWidgetString(WidgetID widget, StringID stringid) const override
 	{
 		switch (widget) {
-			case WID_VT_CAPTION: SetDParam(0, this->vehicle->index); break;
-			case WID_VT_EXPECTED: SetDParam(0, this->show_expected ? STR_TIMETABLE_EXPECTED : STR_TIMETABLE_SCHEDULED); break;
+			case WID_VT_CAPTION: return GetString(STR_TIMETABLE_TITLE, this->vehicle->index);
+			case WID_VT_EXPECTED: return GetString(this->show_expected ? STR_TIMETABLE_EXPECTED : STR_TIMETABLE_SCHEDULED);
+			default: return this->Window::GetWidgetString(widget, stringid);
 		}
 	}
 
+	std::string GetTimetableTravelString(const Order &order, int i, TextColour &colour) const
+	{
+		colour = (i == this->sel_index) ? TC_WHITE : TC_BLACK;
+
+		if (order.IsType(OT_CONDITIONAL)) {
+			return GetString(STR_TIMETABLE_NO_TRAVEL);
+		}
+
+		if (order.IsType(OT_IMPLICIT)) {
+			colour = ((i == this->sel_index) ? TC_SILVER : TC_GREY) | TC_NO_SHADE;
+			return GetString(STR_TIMETABLE_NOT_TIMETABLEABLE);
+		}
+
+		if (order.IsTravelTimetabled()) {
+			auto [str, value] = GetTimetableParameters(order.GetTimetabledTravel());
+			return order.GetMaxSpeed() != UINT16_MAX
+				? GetString(STR_TIMETABLE_TRAVEL_FOR_SPEED, str, value, PackVelocity(order.GetMaxSpeed(), this->vehicle->type))
+				: GetString(STR_TIMETABLE_TRAVEL_FOR, str, value);
+		}
+
+		if (order.GetTravelTime() > 0) {
+			auto [str, value] = GetTimetableParameters(order.GetTravelTime());
+			return order.GetMaxSpeed() != UINT16_MAX
+				? GetString(STR_TIMETABLE_TRAVEL_FOR_SPEED_ESTIMATED, str, value, PackVelocity(order.GetMaxSpeed(), this->vehicle->type))
+				: GetString(STR_TIMETABLE_TRAVEL_FOR_ESTIMATED, str, value);
+		}
+
+		return order.GetMaxSpeed() != UINT16_MAX
+			? GetString(STR_TIMETABLE_TRAVEL_NOT_TIMETABLED_SPEED, PackVelocity(order.GetMaxSpeed(), this->vehicle->type))
+			: GetString(STR_TIMETABLE_TRAVEL_NOT_TIMETABLED);
+	}
+
 	/**
 	 * Helper function to draw the timetable panel.
 	 * @param r The rect to draw within.
@@ -417,6 +426,7 @@ struct TimetableWindow : Window {
 	void DrawTimetablePanel(const Rect &r) const
 	{
 		const Vehicle *v = this->vehicle;
+		if (v->GetNumOrders() == 0) return;
 		Rect tr = r.Shrink(WidgetDimensions::scaled.framerect);
 		int i = this->vscroll->GetPosition();
 		VehicleOrderID order_id = (i + 1) / 2;
@@ -424,51 +434,21 @@ struct TimetableWindow : Window {
 		int selected = this->sel_index;
 
 		bool rtl = _current_text_dir == TD_RTL;
-		SetDParamMaxValue(0, v->GetNumOrders(), 2);
-		int index_column_width = GetStringBoundingBox(STR_ORDER_INDEX).width + 2 * GetSpriteSize(rtl ? SPR_ARROW_RIGHT : SPR_ARROW_LEFT).width + WidgetDimensions::scaled.hsep_normal;
+		int index_column_width = GetStringBoundingBox(GetString(STR_ORDER_INDEX, GetParamMaxValue(v->GetNumOrders(), 2))).width + 2 * GetSpriteSize(rtl ? SPR_ARROW_RIGHT : SPR_ARROW_LEFT).width + WidgetDimensions::scaled.hsep_normal;
 		int middle = rtl ? tr.right - index_column_width : tr.left + index_column_width;
 
-		const Order *order = v->GetOrder(order_id);
-		while (order != nullptr) {
+		auto orders = v->Orders();
+		while (true) {
 			/* Don't draw anything if it extends past the end of the window. */
 			if (!this->vscroll->IsVisible(i)) break;
 
 			if (i % 2 == 0) {
-				DrawOrderString(v, order, order_id, tr.top, i == selected, true, tr.left, middle, tr.right);
-
-				order_id++;
-
-				if (order_id >= v->GetNumOrders()) {
-					order = v->GetOrder(0);
-					final_order = true;
-				} else {
-					order = order->next;
-				}
+				DrawOrderString(v, &orders[order_id], order_id, tr.top, i == selected, true, tr.left, middle, tr.right);
+				if (v->orders->GetNext(order_id) == 0) final_order = true;
+				order_id = v->orders->GetNext(order_id);
 			} else {
-				StringID string;
-				TextColour colour = (i == selected) ? TC_WHITE : TC_BLACK;
-				if (order->IsType(OT_CONDITIONAL)) {
-					string = STR_TIMETABLE_NO_TRAVEL;
-				} else if (order->IsType(OT_IMPLICIT)) {
-					string = STR_TIMETABLE_NOT_TIMETABLEABLE;
-					colour = ((i == selected) ? TC_SILVER : TC_GREY) | TC_NO_SHADE;
-				} else if (!order->IsTravelTimetabled()) {
-					if (order->GetTravelTime() > 0) {
-						SetTimetableParams(0, 1, order->GetTravelTime());
-						string = order->GetMaxSpeed() != UINT16_MAX ?
-							STR_TIMETABLE_TRAVEL_FOR_SPEED_ESTIMATED :
-							STR_TIMETABLE_TRAVEL_FOR_ESTIMATED;
-					} else {
-						string = order->GetMaxSpeed() != UINT16_MAX ?
-							STR_TIMETABLE_TRAVEL_NOT_TIMETABLED_SPEED :
-							STR_TIMETABLE_TRAVEL_NOT_TIMETABLED;
-					}
-				} else {
-					SetTimetableParams(0, 1, order->GetTimetabledTravel());
-					string = order->GetMaxSpeed() != UINT16_MAX ?
-						STR_TIMETABLE_TRAVEL_FOR_SPEED : STR_TIMETABLE_TRAVEL_FOR;
-				}
-				SetDParam(2, PackVelocity(order->GetMaxSpeed(), v->type));
+				TextColour colour;
+				std::string string = GetTimetableTravelString(orders[order_id], i, colour);
 
 				DrawString(rtl ? tr.left : middle, rtl ? middle : tr.right, tr.top, string, colour);
 
@@ -492,16 +472,16 @@ struct TimetableWindow : Window {
 		 * i.e. are only shown if we can calculate all times.
 		 * Excluding order lists with only one order makes some things easier. */
 		TimerGameTick::Ticks total_time = v->orders != nullptr ? v->orders->GetTimetableDurationIncomplete() : 0;
-		if (total_time <= 0 || v->GetNumOrders() <= 1 || !HasBit(v->vehicle_flags, VF_TIMETABLE_STARTED)) return;
+		if (total_time <= 0 || v->GetNumOrders() <= 1 || !v->vehicle_flags.Test(VehicleFlag::TimetableStarted)) return;
 
 		std::vector<TimetableArrivalDeparture> arr_dep(v->GetNumOrders());
 		const VehicleOrderID cur_order = v->cur_real_order_index % v->GetNumOrders();
 
-		VehicleOrderID earlyID = BuildArrivalDepartureList(v, arr_dep) ? cur_order : (VehicleOrderID)INVALID_VEH_ORDER_ID;
+		VehicleOrderID early_id = BuildArrivalDepartureList(v, arr_dep) ? cur_order : (VehicleOrderID)INVALID_VEH_ORDER_ID;
 		int selected = this->sel_index;
 
 		Rect tr = r.Shrink(WidgetDimensions::scaled.framerect);
-		bool show_late = this->show_expected && VehicleIsAboveLatenessThreshold(v, true);
+		bool show_late = this->show_expected && VehicleIsAboveLatenessThreshold(v->lateness_counter, true);
 		TimerGameTick::Ticks offset = show_late ? 0 : -v->lateness_counter;
 
 		for (int i = this->vscroll->GetPosition(); i / 2 < v->GetNumOrders(); ++i) { // note: i is also incremented in the loop
@@ -509,16 +489,16 @@ struct TimetableWindow : Window {
 			if (!this->vscroll->IsVisible(i)) break;
 
 			/* TC_INVALID will skip the colour change. */
-			SetDParam(0, show_late ? TC_RED : TC_INVALID);
+			TextColour tc = show_late ? TC_RED : TC_INVALID;
 			if (i % 2 == 0) {
 				/* Draw an arrival time. */
 				if (arr_dep[i / 2].arrival != Ticks::INVALID_TICKS) {
 					/* First set the offset and text colour based on the expected/scheduled mode and some other things. */
 					TimerGameTick::Ticks this_offset;
-					if (this->show_expected && i / 2 == earlyID) {
+					if (this->show_expected && i / 2 == early_id) {
 						/* Show expected arrival. */
 						this_offset = 0;
-						SetDParam(0, TC_GREEN);
+						tc = TC_GREEN;
 					} else {
 						/* Show scheduled arrival. */
 						this_offset = offset;
@@ -527,12 +507,14 @@ struct TimetableWindow : Window {
 					/* Now actually draw the arrival time. */
 					if (_settings_client.gui.timetable_mode == TimetableMode::Seconds) {
 						/* Display seconds from now. */
-						SetDParam(1, ((arr_dep[i / 2].arrival + offset) / Ticks::TICKS_PER_SECOND));
-						DrawString(tr.left, tr.right, tr.top, STR_TIMETABLE_ARRIVAL_SECONDS_IN_FUTURE, i == selected ? TC_WHITE : TC_BLACK);
+						DrawString(tr,
+							GetString(STR_TIMETABLE_ARRIVAL_SECONDS_IN_FUTURE, tc, (arr_dep[i / 2].arrival + offset) / Ticks::TICKS_PER_SECOND),
+							i == selected ? TC_WHITE : TC_BLACK);
 					} else {
 						/* Show a date. */
-						SetDParam(1, TimerGameEconomy::date + (arr_dep[i / 2].arrival + this_offset) / Ticks::DAY_TICKS);
-						DrawString(tr.left, tr.right, tr.top, STR_TIMETABLE_ARRIVAL_DATE, i == selected ? TC_WHITE : TC_BLACK);
+						DrawString(tr,
+							GetString(STR_TIMETABLE_ARRIVAL_DATE, tc, TimerGameEconomy::date + (arr_dep[i / 2].arrival + this_offset) / Ticks::DAY_TICKS),
+							i == selected ? TC_WHITE : TC_BLACK);
 					}
 				}
 			} else {
@@ -540,12 +522,14 @@ struct TimetableWindow : Window {
 				if (arr_dep[i / 2].departure != Ticks::INVALID_TICKS) {
 					if (_settings_client.gui.timetable_mode == TimetableMode::Seconds) {
 						/* Display seconds from now. */
-						SetDParam(1, ((arr_dep[i / 2].departure + offset) / Ticks::TICKS_PER_SECOND));
-						DrawString(tr.left, tr.right, tr.top, STR_TIMETABLE_DEPARTURE_SECONDS_IN_FUTURE, i == selected ? TC_WHITE : TC_BLACK);
+						DrawString(tr,
+							GetString(STR_TIMETABLE_DEPARTURE_SECONDS_IN_FUTURE, tc, (arr_dep[i / 2].departure + offset) / Ticks::TICKS_PER_SECOND),
+							i == selected ? TC_WHITE : TC_BLACK);
 					} else {
 						/* Show a date. */
-						SetDParam(1, TimerGameEconomy::date + (arr_dep[i / 2].departure + offset) / Ticks::DAY_TICKS);
-						DrawString(tr.left, tr.right, tr.top, STR_TIMETABLE_DEPARTURE_DATE, i == selected ? TC_WHITE : TC_BLACK);
+						DrawString(tr,
+							GetString(STR_TIMETABLE_DEPARTURE_DATE, tc, TimerGameEconomy::date + (arr_dep[i / 2].departure + offset) / Ticks::DAY_TICKS),
+							i == selected ? TC_WHITE : TC_BLACK);
 					}
 				}
 			}
@@ -553,6 +537,14 @@ struct TimetableWindow : Window {
 		}
 	}
 
+	std::string GetTimetableTotalTimeString(TimerGameTick::Ticks total_time) const
+	{
+		auto [str, value] = GetTimetableParameters(total_time);
+		if (!this->vehicle->orders->IsCompleteTimetable()) return GetString(STR_TIMETABLE_TOTAL_TIME_INCOMPLETE, str, value);
+		if (total_time % TicksPerTimetableUnit() == 0) return GetString(STR_TIMETABLE_TOTAL_TIME, str, value);
+		return GetString(STR_TIMETABLE_APPROX_TIME, str, value);
+	}
+
 	/**
 	 * Helper function to draw the summary panel.
 	 * @param r The rect to draw within.
@@ -564,8 +556,7 @@ struct TimetableWindow : Window {
 
 		TimerGameTick::Ticks total_time = v->orders != nullptr ? v->orders->GetTimetableDurationIncomplete() : 0;
 		if (total_time != 0) {
-			SetTimetableParams(0, 1, total_time);
-			DrawString(tr, v->orders->IsCompleteTimetable() ? STR_TIMETABLE_TOTAL_TIME : STR_TIMETABLE_TOTAL_TIME_INCOMPLETE);
+			DrawString(tr, GetTimetableTotalTimeString(total_time));
 		}
 		tr.top += GetCharacterHeight(FS_NORMAL);
 
@@ -575,24 +566,21 @@ struct TimetableWindow : Window {
 			 * timetable at the given time. */
 			if (_settings_client.gui.timetable_mode == TimetableMode::Seconds) {
 				/* Real time units use seconds relative to now. */
-				SetDParam(0, (static_cast<TimerGameTick::Ticks>(v->timetable_start - TimerGameTick::counter) / Ticks::TICKS_PER_SECOND));
-				DrawString(tr, STR_TIMETABLE_STATUS_START_IN_SECONDS);
+				DrawString(tr, GetString(STR_TIMETABLE_STATUS_START_IN_SECONDS, static_cast<TimerGameTick::Ticks>(v->timetable_start - TimerGameTick::counter) / Ticks::TICKS_PER_SECOND));
 			} else {
 				/* Other units use dates. */
-				SetDParam(0, STR_JUST_DATE_TINY);
-				SetDParam(1, GetDateFromStartTick(v->timetable_start));
-				DrawString(tr, STR_TIMETABLE_STATUS_START_AT_DATE);
+				DrawString(tr, GetString(STR_TIMETABLE_STATUS_START_AT_DATE, STR_JUST_DATE_TINY, GetDateFromStartTick(v->timetable_start)));
 			}
-		} else if (!HasBit(v->vehicle_flags, VF_TIMETABLE_STARTED)) {
+		} else if (!v->vehicle_flags.Test(VehicleFlag::TimetableStarted)) {
 			/* We aren't running on a timetable yet. */
 			DrawString(tr, STR_TIMETABLE_STATUS_NOT_STARTED);
-		} else if (!VehicleIsAboveLatenessThreshold(v, false)) {
+		} else if (!VehicleIsAboveLatenessThreshold(abs(v->lateness_counter), false)) {
 			/* We are on time. */
 			DrawString(tr, STR_TIMETABLE_STATUS_ON_TIME);
 		} else {
 			/* We are late. */
-			SetTimetableParams(0, 1, abs(v->lateness_counter));
-			DrawString(tr, v->lateness_counter < 0 ? STR_TIMETABLE_STATUS_EARLY : STR_TIMETABLE_STATUS_LATE);
+			auto [str, value] = GetTimetableParameters(abs(v->lateness_counter));
+			DrawString(tr, GetString(v->lateness_counter < 0 ? STR_TIMETABLE_STATUS_EARLY : STR_TIMETABLE_STATUS_LATE, str, value));
 		}
 	}
 
@@ -639,7 +627,7 @@ struct TimetableWindow : Window {
 				int selected = GetOrderFromTimetableWndPt(pt.y, v);
 
 				this->CloseChildWindows();
-				this->sel_index = (selected == INVALID_ORDER || selected == this->sel_index) ? -1 : selected;
+				this->sel_index = (selected == INT32_MAX || selected == this->sel_index) ? -1 : selected;
 				break;
 			}
 
@@ -647,9 +635,13 @@ struct TimetableWindow : Window {
 				if (_settings_client.gui.timetable_mode == TimetableMode::Seconds) {
 					this->query_widget = WID_VT_START_DATE;
 					this->change_timetable_all = _ctrl_pressed;
-					ShowQueryString(STR_EMPTY, STR_TIMETABLE_START_SECONDS_QUERY, 6, this, CS_NUMERAL, QSF_ACCEPT_UNCHANGED);
+					ShowQueryString({}, STR_TIMETABLE_START_SECONDS_QUERY, 6, this, CS_NUMERAL, QueryStringFlag::AcceptUnchanged);
 				} else {
-					ShowSetDateWindow(this, v->index, TimerGameEconomy::date, TimerGameEconomy::year, TimerGameEconomy::year + MAX_TIMETABLE_START_YEARS, ChangeTimetableStartCallback, reinterpret_cast<void*>(static_cast<uintptr_t>(_ctrl_pressed)));
+					ShowSetDateWindow(this, v->index.base(), TimerGameEconomy::date, TimerGameEconomy::year, TimerGameEconomy::year + MAX_TIMETABLE_START_YEARS,
+						[ctrl=_ctrl_pressed](const Window *w, TimerGameEconomy::Date date) {
+							Command<CMD_SET_TIMETABLE_START>::Post(STR_ERROR_CAN_T_TIMETABLE_VEHICLE, w->window_number, ctrl, GetStartTickFromDate(date));
+						}
+					);
 				}
 				break;
 
@@ -661,20 +653,19 @@ struct TimetableWindow : Window {
 				if (real >= v->GetNumOrders()) real = 0;
 
 				const Order *order = v->GetOrder(real);
-				StringID current = STR_EMPTY;
+				std::string current;
 
 				if (order != nullptr) {
 					uint time = (selected % 2 != 0) ? order->GetTravelTime() : order->GetWaitTime();
 					time /= TicksPerTimetableUnit();
 
 					if (time != 0) {
-						SetDParam(0, time);
-						current = STR_JUST_INT;
+						current = GetString(STR_JUST_INT, time);
 					}
 				}
 
 				this->change_timetable_all = _ctrl_pressed && (order != nullptr);
-				ShowQueryString(current, STR_TIMETABLE_CHANGE_TIME, 31, this, CS_NUMERAL, QSF_ACCEPT_UNCHANGED);
+				ShowQueryString(current, STR_TIMETABLE_CHANGE_TIME_QUERY, 31, this, CS_NUMERAL, QueryStringFlag::AcceptUnchanged);
 				break;
 			}
 
@@ -685,17 +676,16 @@ struct TimetableWindow : Window {
 
 				if (real >= v->GetNumOrders()) real = 0;
 
-				StringID current = STR_EMPTY;
+				std::string current;
 				const Order *order = v->GetOrder(real);
 				if (order != nullptr) {
 					if (order->GetMaxSpeed() != UINT16_MAX) {
-						SetDParam(0, ConvertKmhishSpeedToDisplaySpeed(order->GetMaxSpeed(), v->type));
-						current = STR_JUST_INT;
+						current = GetString(STR_JUST_INT, ConvertKmhishSpeedToDisplaySpeed(order->GetMaxSpeed(), v->type));
 					}
 				}
 
 				this->change_timetable_all = _ctrl_pressed && (order != nullptr);
-				ShowQueryString(current, STR_TIMETABLE_CHANGE_SPEED, 31, this, CS_NUMERAL, QSF_NONE);
+				ShowQueryString(current, STR_TIMETABLE_CHANGE_SPEED_QUERY, 31, this, CS_NUMERAL, {});
 				break;
 			}
 
@@ -724,7 +714,7 @@ struct TimetableWindow : Window {
 				break;
 
 			case WID_VT_AUTOFILL: { // Autofill the timetable.
-				Command<CMD_AUTOFILL_TIMETABLE>::Post(STR_ERROR_CAN_T_TIMETABLE_VEHICLE, v->index, !HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE), _ctrl_pressed);
+				Command<CMD_AUTOFILL_TIMETABLE>::Post(STR_ERROR_CAN_T_TIMETABLE_VEHICLE, v->index, !v->vehicle_flags.Test(VehicleFlag::AutofillTimetable), _ctrl_pressed);
 				break;
 			}
 
@@ -740,12 +730,12 @@ struct TimetableWindow : Window {
 		this->SetDirty();
 	}
 
-	void OnQueryTextFinished(char *str) override
+	void OnQueryTextFinished(std::optional<std::string> str) override
 	{
-		if (str == nullptr) return;
+		if (!str.has_value()) return;
 
 		const Vehicle *v = this->vehicle;
-		uint64_t val = StrEmpty(str) ? 0 : std::strtoul(str, nullptr, 10);
+		uint64_t val = ParseInteger<uint64_t>(*str, 10, true).value_or(0);
 		auto [order_id, mtf] = PackTimetableArgs(v, this->sel_index, query_widget == WID_VT_CHANGE_SPEED);
 
 		switch (query_widget) {
@@ -799,64 +789,64 @@ struct TimetableWindow : Window {
 	/**
 	 * In real-time mode, the timetable GUI shows relative times and needs to be redrawn every second.
 	 */
-	IntervalTimer<TimerGameTick> redraw_interval = { { TimerGameTick::Priority::NONE, Ticks::TICKS_PER_SECOND }, [this](auto) {
+	const IntervalTimer<TimerGameTick> redraw_interval = { { TimerGameTick::Priority::NONE, Ticks::TICKS_PER_SECOND }, [this](auto) {
 		if (_settings_client.gui.timetable_mode == TimetableMode::Seconds) {
 			this->SetDirty();
 		}
 	}};
 };
 
-static constexpr NWidgetPart _nested_timetable_widgets[] = {
+static constexpr std::initializer_list<NWidgetPart> _nested_timetable_widgets = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
-		NWidget(WWT_CAPTION, COLOUR_GREY, WID_VT_CAPTION), SetDataTip(STR_TIMETABLE_TITLE, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_ORDER_VIEW), SetMinimalSize(61, 14), SetDataTip( STR_TIMETABLE_ORDER_VIEW, STR_TIMETABLE_ORDER_VIEW_TOOLTIP),
+		NWidget(WWT_CAPTION, COLOUR_GREY, WID_VT_CAPTION),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_ORDER_VIEW), SetMinimalSize(61, 14), SetStringTip(STR_TIMETABLE_ORDER_VIEW, STR_TIMETABLE_ORDER_VIEW_TOOLTIP),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_DEFSIZEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_TIMETABLE_PANEL), SetMinimalSize(388, 82), SetResize(1, 10), SetDataTip(STR_NULL, STR_TIMETABLE_TOOLTIP), SetScrollbar(WID_VT_SCROLLBAR), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_TIMETABLE_PANEL), SetMinimalSize(388, 82), SetResize(1, 10), SetToolTip(STR_TIMETABLE_TOOLTIP), SetScrollbar(WID_VT_SCROLLBAR), EndContainer(),
 		NWidget(NWID_SELECTION, INVALID_COLOUR, WID_VT_ARRIVAL_DEPARTURE_SELECTION),
-			NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_ARRIVAL_DEPARTURE_PANEL), SetMinimalSize(110, 0), SetFill(0, 1), SetDataTip(STR_NULL, STR_TIMETABLE_TOOLTIP), SetScrollbar(WID_VT_SCROLLBAR), EndContainer(),
+			NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_ARRIVAL_DEPARTURE_PANEL), SetMinimalSize(110, 0), SetFill(0, 1), SetToolTip(STR_TIMETABLE_TOOLTIP), SetScrollbar(WID_VT_SCROLLBAR), EndContainer(),
 		EndContainer(),
 		NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_VT_SCROLLBAR),
 	EndContainer(),
 	NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_SUMMARY_PANEL), SetMinimalSize(400, 22), SetResize(1, 0), EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(NWID_HORIZONTAL, NC_EQUALSIZE),
-			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CHANGE_TIME), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CHANGE_TIME, STR_TIMETABLE_WAIT_TIME_TOOLTIP),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CLEAR_TIME), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CLEAR_TIME, STR_TIMETABLE_CLEAR_TIME_TOOLTIP),
+		NWidget(NWID_HORIZONTAL, NWidContainerFlag::EqualSize),
+			NWidget(NWID_VERTICAL, NWidContainerFlag::EqualSize),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CHANGE_TIME), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_CHANGE_TIME, STR_TIMETABLE_WAIT_TIME_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CLEAR_TIME), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_CLEAR_TIME, STR_TIMETABLE_CLEAR_TIME_TOOLTIP),
 			EndContainer(),
-			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CHANGE_SPEED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CHANGE_SPEED, STR_TIMETABLE_CHANGE_SPEED_TOOLTIP),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CLEAR_SPEED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CLEAR_SPEED, STR_TIMETABLE_CLEAR_SPEED_TOOLTIP),
+			NWidget(NWID_VERTICAL, NWidContainerFlag::EqualSize),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CHANGE_SPEED), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_CHANGE_SPEED, STR_TIMETABLE_CHANGE_SPEED_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CLEAR_SPEED), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_CLEAR_SPEED, STR_TIMETABLE_CLEAR_SPEED_TOOLTIP),
 			EndContainer(),
-			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_START_DATE), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_START, STR_TIMETABLE_START_TOOLTIP),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_RESET_LATENESS), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_RESET_LATENESS, STR_TIMETABLE_RESET_LATENESS_TOOLTIP),
+			NWidget(NWID_VERTICAL, NWidContainerFlag::EqualSize),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_START_DATE), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_START, STR_TIMETABLE_START_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_RESET_LATENESS), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_RESET_LATENESS, STR_TIMETABLE_RESET_LATENESS_TOOLTIP),
 			EndContainer(),
-			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_AUTOFILL), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_AUTOFILL, STR_TIMETABLE_AUTOFILL_TOOLTIP),
+			NWidget(NWID_VERTICAL, NWidContainerFlag::EqualSize),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_AUTOFILL), SetResize(1, 0), SetFill(1, 1), SetStringTip(STR_TIMETABLE_AUTOFILL, STR_TIMETABLE_AUTOFILL_TOOLTIP),
 				NWidget(NWID_SELECTION, INVALID_COLOUR, WID_VT_EXPECTED_SELECTION),
-					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_EXPECTED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_JUST_STRING, STR_TIMETABLE_EXPECTED_TOOLTIP),
+					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_EXPECTED), SetResize(1, 0), SetFill(1, 1), SetToolTip(STR_TIMETABLE_EXPECTED_TOOLTIP),
 					NWidget(WWT_PANEL, COLOUR_GREY), SetResize(1, 0), SetFill(1, 1), EndContainer(),
 				EndContainer(),
 			EndContainer(),
 		EndContainer(),
-		NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-			NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_VT_SHARED_ORDER_LIST), SetFill(0, 1), SetDataTip(SPR_SHARED_ORDERS_ICON, STR_ORDERS_VEH_WITH_SHARED_ORDERS_LIST_TOOLTIP),
+		NWidget(NWID_VERTICAL, NWidContainerFlag::EqualSize),
+			NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_VT_SHARED_ORDER_LIST), SetFill(0, 1), SetSpriteTip(SPR_SHARED_ORDERS_ICON, STR_ORDERS_VEH_WITH_SHARED_ORDERS_LIST_TOOLTIP),
 			NWidget(WWT_RESIZEBOX, COLOUR_GREY), SetFill(0, 1),
 		EndContainer(),
 	EndContainer(),
 };
 
-static WindowDesc _timetable_desc(__FILE__, __LINE__,
+static WindowDesc _timetable_desc(
 	WDP_AUTO, "view_vehicle_timetable", 400, 130,
 	WC_VEHICLE_TIMETABLE, WC_VEHICLE_VIEW,
-	WDF_CONSTRUCTION,
-	std::begin(_nested_timetable_widgets), std::end(_nested_timetable_widgets)
+	WindowDefaultFlag::Construction,
+	_nested_timetable_widgets
 );
 
 /**
@@ -867,5 +857,5 @@ void ShowTimetableWindow(const Vehicle *v)
 {
 	CloseWindowById(WC_VEHICLE_DETAILS, v->index, false);
 	CloseWindowById(WC_VEHICLE_ORDERS, v->index, false);
-	AllocateWindowDescFront<TimetableWindow>(&_timetable_desc, v->index);
+	AllocateWindowDescFront<TimetableWindow>(_timetable_desc, v->index);
 }
