@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file cocoa_v.h The Cocoa video driver. */
@@ -30,7 +30,7 @@ public:
 
 	OTTD_CocoaWindow *window;    ///< Pointer to window object
 	OTTD_CocoaView *cocoaview;   ///< Pointer to view object
-	CGColorSpaceRef color_space; ///< Window color space
+	CGColorSpaceRef colour_space; ///< Window colour space
 
 	OTTD_CocoaWindowDelegate *delegate; //!< Window delegate object
 
@@ -64,7 +64,6 @@ protected:
 	bool buffer_locked; ///< Video buffer was locked by the main thread.
 
 	Dimension GetScreenSize() const override;
-	float GetDPIScale() override;
 	void InputLoop() override;
 	bool LockVideoBuffer() override;
 	void UnlockVideoBuffer() override;
@@ -72,7 +71,7 @@ protected:
 
 	void GameSizeChanged();
 
-	const char *Initialize();
+	std::optional<std::string_view> Initialize();
 
 	void UpdateVideoModes();
 
@@ -92,8 +91,8 @@ private:
 class VideoDriver_CocoaQuartz : public VideoDriver_Cocoa {
 private:
 	int buffer_depth;     ///< Colour depth of used frame buffer
-	void *pixel_buffer;   ///< used for direct pixel access
-	void *window_buffer;  ///< Colour translation from palette to screen
+	std::unique_ptr<uint8_t[]> pixel_buffer; ///< used for direct pixel access
+	std::unique_ptr<uint32_t[]> window_buffer; ///< Colour translation from palette to screen
 
 	int window_width;     ///< Current window width in pixel
 	int window_height;    ///< Current window height in pixel
@@ -102,18 +101,18 @@ private:
 	uint32_t palette[256];  ///< Colour Palette
 
 	void BlitIndexedToView32(int left, int top, int right, int bottom);
-	void UpdatePalette(uint first_color, uint num_colors);
+	void UpdatePalette(uint first_colour, uint num_colours);
 
 public:
 	CGContextRef cgcontext;      ///< Context reference for Quartz subdriver
 
 	VideoDriver_CocoaQuartz();
 
-	const char *Start(const StringList &param) override;
+	std::optional<std::string_view> Start(const StringList &param) override;
 	void Stop() override;
 
 	/** Return driver name */
-	const char *GetName() const override { return "cocoa"; }
+	std::string_view GetName() const override { return "cocoa"; }
 
 	void AllocateBackingStore(bool force = false) override;
 
@@ -123,13 +122,13 @@ protected:
 
 	NSView *AllocateDrawView() override;
 
-	void *GetVideoPointer() override { return this->buffer_depth == 8 ? this->pixel_buffer : this->window_buffer; }
+	void *GetVideoPointer() override { return this->buffer_depth == 8 ? static_cast<void *>(this->pixel_buffer.get()) : static_cast<void *>(this->window_buffer.get()); }
 };
 
 class FVideoDriver_CocoaQuartz : public DriverFactoryBase {
 public:
 	FVideoDriver_CocoaQuartz() : DriverFactoryBase(Driver::DT_VIDEO, 8, "cocoa", "Cocoa Video Driver") {}
-	Driver *CreateInstance() const override { return new VideoDriver_CocoaQuartz(); }
+	std::unique_ptr<Driver> CreateInstance() const override { return std::make_unique<VideoDriver_CocoaQuartz>(); }
 };
 
 #endif /* VIDEO_COCOA_H */
