@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file opengl_v.cpp OpenGL video driver support. */
@@ -29,13 +29,12 @@
 
 #include "opengl.h"
 #include "../core/geometry_func.hpp"
-#include "../core/mem_func.hpp"
 #include "../core/math_func.hpp"
-#include "../core/mem_func.hpp"
 #include "../gfx_func.h"
 #include "../debug.h"
 #include "../blitter/factory.hpp"
 #include "../zoom_func.h"
+#include "../core/string_consumer.hpp"
 
 #include "../table/opengl_shader.h"
 #include "../table/sprites.h"
@@ -134,30 +133,28 @@ static const int MAX_CACHED_CURSORS = 48;
 
 GetOGLProcAddressProc GetOGLProcAddress;
 
+static std::optional<std::string_view> GlGetString(GLenum name)
+{
+	auto str = reinterpret_cast<const char *>(_glGetString(name));
+	if (str == nullptr) return {};
+	return str;
+}
+
 /**
  * Find a substring in a string made of space delimited elements. The substring
  * has to match the complete element, partial matches don't count.
  * @param string List of space delimited elements.
  * @param substring Substring to find.
- * @return Pointer to the start of the match or nullptr if the substring is not present.
+ * @return Whether the substring was found.
  */
-const char *FindStringInExtensionList(const char *string, const char *substring)
+bool HasStringInExtensionList(std::string_view string, std::string_view substring)
 {
-	while (true) {
-		/* Is the extension string present at all? */
-		const char *pos = strstr(string, substring);
-		if (pos == nullptr) break;
-
-		/* Is this a real match, i.e. are the chars before and after the matched string
-		 * indeed spaces (or the start or end of the string, respectively)? */
-		const char *end = pos + strlen(substring);
-		if ((pos == string || pos[-1] == ' ') && (*end == ' ' || *end == '\0')) return pos;
-
-		/* False hit, try again for the remaining string. */
-		string = end;
+	StringConsumer consumer{string};
+	while (consumer.AnyBytesLeft()) {
+		if (substring == consumer.ReadUntil(" ", StringConsumer::SKIP_ALL_SEPARATORS)) return true;
 	}
 
-	return nullptr;
+	return false;
 }
 
 /**
@@ -165,7 +162,7 @@ const char *FindStringInExtensionList(const char *string, const char *substring)
  * @param extension The extension string to test.
  * @return True if the extension is supported, false if not.
  */
-static bool IsOpenGLExtensionSupported(const char *extension)
+static bool IsOpenGLExtensionSupported(std::string_view extension)
 {
 	static PFNGLGETSTRINGIPROC glGetStringi = nullptr;
 	static bool glGetStringi_loaded = false;
@@ -183,19 +180,19 @@ static bool IsOpenGLExtensionSupported(const char *extension)
 		_glGetIntegerv(GL_NUM_EXTENSIONS, &num_exts);
 
 		for (GLint i = 0; i < num_exts; i++) {
-			const char *entry = (const char *)glGetStringi(GL_EXTENSIONS, i);
-			if (strcmp(entry, extension) == 0) return true;
+			const char *entry = reinterpret_cast<const char *>(glGetStringi(GL_EXTENSIONS, i));
+			if (entry != nullptr && entry == extension) return true;
 		}
-	} else {
+	} else if (auto str = GlGetString(GL_EXTENSIONS); str.has_value()) {
 		/* Old style: A single, space-delimited string for all extensions. */
-		return FindStringInExtensionList((const char *)_glGetString(GL_EXTENSIONS), extension) != nullptr;
+		return HasStringInExtensionList(*str, extension);
 	}
 
 	return false;
 }
 
-static byte _gl_major_ver = 0; ///< Major OpenGL version.
-static byte _gl_minor_ver = 0; ///< Minor OpenGL version.
+static uint8_t _gl_major_ver = 0; ///< Major OpenGL version.
+static uint8_t _gl_minor_ver = 0; ///< Minor OpenGL version.
 
 /**
  * Check if the current OpenGL version is equal or higher than a given one.
@@ -204,7 +201,7 @@ static byte _gl_minor_ver = 0; ///< Minor OpenGL version.
  * @pre OpenGL was initialized.
  * @return True if the OpenGL version is equal or higher than the requested one.
  */
-bool IsOpenGLVersionAtLeast(byte major, byte minor)
+bool IsOpenGLVersionAtLeast(uint8_t major, uint8_t minor)
 {
 	return (_gl_major_ver > major) || (_gl_major_ver == major && _gl_minor_ver >= minor);
 }
@@ -408,7 +405,7 @@ static bool BindPersistentBufferExtensions()
 void APIENTRY DebugOutputCallback([[maybe_unused]] GLenum source, GLenum type, [[maybe_unused]] GLuint id, GLenum severity, [[maybe_unused]] GLsizei length, const GLchar *message, [[maybe_unused]] const void *userParam)
 {
 	/* Make severity human readable. */
-	const char *severity_str = "";
+	std::string_view severity_str;
 	switch (severity) {
 		case GL_DEBUG_SEVERITY_HIGH:   severity_str = "high"; break;
 		case GL_DEBUG_SEVERITY_MEDIUM: severity_str = "medium"; break;
@@ -416,7 +413,7 @@ void APIENTRY DebugOutputCallback([[maybe_unused]] GLenum source, GLenum type, [
 	}
 
 	/* Make type human readable.*/
-	const char *type_str = "Other";
+	std::string_view type_str = "Other";
 	switch (type) {
 		case GL_DEBUG_TYPE_ERROR:               type_str = "Error"; break;
 		case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR: type_str = "Deprecated"; break;
@@ -462,9 +459,9 @@ void SetupDebugOutput()
  * Create and initialize the singleton back-end class.
  * @param get_proc Callback to get an OpenGL function from the OS driver.
  * @param screen_res Current display resolution.
- * @return nullptr on success, error message otherwise.
+ * @return std::nullopt on success, error message otherwise.
  */
-/* static */ const char *OpenGLBackend::Create(GetOGLProcAddressProc get_proc, const Dimension &screen_res)
+/* static */ std::optional<std::string_view> OpenGLBackend::Create(GetOGLProcAddressProc get_proc, const Dimension &screen_res)
 {
 	if (OpenGLBackend::instance != nullptr) OpenGLBackend::Destroy();
 
@@ -517,33 +514,39 @@ OpenGLBackend::~OpenGLBackend()
 	}
 }
 
+static std::tuple<uint8_t, uint8_t> DecodeVersion(std::string_view ver)
+{
+	StringConsumer consumer{ver};
+	int major = consumer.ReadIntegerBase<uint8_t>(10);
+	if (consumer.ReadIf(".")) return {major, consumer.ReadIntegerBase<uint8_t>(10)};
+	return {major, 0};
+}
+
 /**
  * Check for the needed OpenGL functionality and allocate all resources.
  * @param screen_res Current display resolution.
- * @return Error string or nullptr if successful.
+ * @return Error string or std::nullopt if successful.
  */
-const char *OpenGLBackend::Init(const Dimension &screen_res)
+std::optional<std::string_view> OpenGLBackend::Init(const Dimension &screen_res)
 {
 	if (!BindBasicInfoProcs()) return "OpenGL not supported";
 
 	/* Always query the supported OpenGL version as the current context might have changed. */
-	const char *ver = (const char *)_glGetString(GL_VERSION);
-	const char *vend = (const char *)_glGetString(GL_VENDOR);
-	const char *renderer = (const char *)_glGetString(GL_RENDERER);
+	auto ver = GlGetString(GL_VERSION);
+	auto vend = GlGetString(GL_VENDOR);
+	auto renderer = GlGetString(GL_RENDERER);
 
-	if (ver == nullptr || vend == nullptr || renderer == nullptr) return "OpenGL not supported";
+	if (!ver.has_value() || !vend.has_value() || !renderer.has_value()) return "OpenGL not supported";
 
-	Debug(driver, 1, "OpenGL driver: {} - {} ({})", vend, renderer, ver);
+	Debug(driver, 1, "OpenGL driver: {} - {} ({})", *vend, *renderer, *ver);
 
 #ifndef GL_ALLOW_SOFTWARE_RENDERER
 	/* Don't use MESA software rendering backends as they are slower than
 	 * just using a non-OpenGL video driver. */
-	if (strncmp(renderer, "llvmpipe", 8) == 0 || strncmp(renderer, "softpipe", 8) == 0) return "Software renderer detected, not using OpenGL";
+	if (renderer->starts_with("llvmpipe") || renderer->starts_with("softpipe")) return "Software renderer detected, not using OpenGL";
 #endif
 
-	const char *minor = strchr(ver, '.');
-	_gl_major_ver = atoi(ver);
-	_gl_minor_ver = minor != nullptr ? atoi(minor + 1) : 0;
+	std::tie(_gl_major_ver, _gl_minor_ver) = DecodeVersion(*ver);
 
 #ifdef _WIN32
 	/* Old drivers on Windows (especially if made by Intel) seem to be
@@ -596,7 +599,7 @@ const char *OpenGLBackend::Init(const Dimension &screen_res)
 	_glGetIntegerv(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &max_tex_units);
 	if (max_tex_units < 4) return "Not enough simultaneous textures supported";
 
-	Debug(driver, 2, "OpenGL shading language version: {}, texture units = {}", (const char *)_glGetString(GL_SHADING_LANGUAGE_VERSION), (int)max_tex_units);
+	Debug(driver, 2, "OpenGL shading language version: {}, texture units = {}", GlGetString(GL_SHADING_LANGUAGE_VERSION).value_or("Unknown version"), max_tex_units);
 
 	if (!this->InitShaders()) return "Failed to initialize shaders";
 
@@ -697,7 +700,7 @@ const char *OpenGLBackend::Init(const Dimension &screen_res)
 	/* Prime vertex buffer with a full-screen quad and store
 	 * the corresponding state in a vertex array object. */
 	static const Simple2DVertex vert_array[] = {
-		//  x     y    u    v
+		/*  x     y    u    v */
 		{  1.f, -1.f, 1.f, 1.f },
 		{  1.f,  1.f, 1.f, 0.f },
 		{ -1.f, -1.f, 0.f, 1.f },
@@ -729,7 +732,7 @@ const char *OpenGLBackend::Init(const Dimension &screen_res)
 	this->PrepareContext();
 	(void)_glGetError(); // Clear errors.
 
-	return nullptr;
+	return std::nullopt;
 }
 
 void OpenGLBackend::PrepareContext()
@@ -743,12 +746,10 @@ void OpenGLBackend::PrepareContext()
 
 std::string OpenGLBackend::GetDriverName()
 {
-	std::string res{};
+	auto renderer = GlGetString(GL_RENDERER);
+	auto version = GlGetString(GL_VERSION);
 	/* Skipping GL_VENDOR as it tends to be "obvious" from the renderer and version data, and just makes the string pointlessly longer */
-	res += reinterpret_cast<const char *>(_glGetString(GL_RENDERER));
-	res += ", ";
-	res += reinterpret_cast<const char *>(_glGetString(GL_VERSION));
-	return res;
+	return fmt::format("{}, {}", renderer.value_or("Unknown renderer"), version.value_or("Unknown version"));
 }
 
 /**
@@ -803,11 +804,10 @@ static bool VerifyProgram(GLuint program)
  */
 bool OpenGLBackend::InitShaders()
 {
-	const char *ver = (const char *)_glGetString(GL_SHADING_LANGUAGE_VERSION);
-	if (ver == nullptr) return false;
+	auto ver = GlGetString(GL_SHADING_LANGUAGE_VERSION);
+	if (!ver.has_value()) return false;
 
-	int glsl_major  = ver[0] - '0';
-	int glsl_minor = ver[2] - '0';
+	auto [glsl_major, glsl_minor] = DecodeVersion(*ver);
 
 	bool glsl_150 = (IsOpenGLVersionAtLeast(3, 2) || glsl_major > 1 || (glsl_major == 1 && glsl_minor >= 5)) && _glBindFragDataLocation != nullptr;
 
@@ -944,10 +944,10 @@ bool OpenGLBackend::Resize(int w, int h, bool force)
 		}
 	} else if (bpp == 8) {
 		if (_glClearBufferSubData != nullptr) {
-			byte b = 0;
+			uint8_t b = 0;
 			_glClearBufferSubData(GL_PIXEL_UNPACK_BUFFER, GL_R8, 0, line_pixel_count, GL_RED, GL_UNSIGNED_BYTE, &b);
 		} else {
-			ClearPixelBuffer<byte>(line_pixel_count, 0);
+			ClearPixelBuffer<uint8_t>(line_pixel_count, 0);
 		}
 	}
 
@@ -975,10 +975,10 @@ bool OpenGLBackend::Resize(int w, int h, bool force)
 
 		/* Initialize buffer as 0 == no remap. */
 		if (_glClearBufferSubData != nullptr) {
-			byte b = 0;
+			uint8_t b = 0;
 			_glClearBufferSubData(GL_PIXEL_UNPACK_BUFFER, GL_R8, 0, line_pixel_count, GL_RED, GL_UNSIGNED_BYTE, &b);
 		} else {
-			ClearPixelBuffer<byte>(line_pixel_count, 0);
+			ClearPixelBuffer<uint8_t>(line_pixel_count, 0);
 		}
 
 		_glBindTexture(GL_TEXTURE_2D, this->anim_texture);
@@ -1074,26 +1074,31 @@ void OpenGLBackend::DrawMouseCursor()
 
 	/* Draw cursor on screen */
 	_cur_dpi = &_screen;
-	for (uint i = 0; i < this->cursor_sprite_count; ++i) {
-		SpriteID sprite = this->cursor_sprite_seq[i].sprite;
-
+	for (const auto &cs : this->cursor_sprites) {
 		/* Sprites are cached by PopulateCursorCache(). */
-		if (this->cursor_cache.Contains(sprite)) {
-			Sprite *spr = this->cursor_cache.Get(sprite);
+		if (this->cursor_cache.Contains(cs.image.sprite)) {
+			const OpenGLSprite *spr = this->cursor_cache.Get(cs.image.sprite).get();
 
-			this->RenderOglSprite((OpenGLSprite *)spr->data, this->cursor_sprite_seq[i].pal,
-					this->cursor_pos.x + this->cursor_sprite_pos[i].x + UnScaleByZoom(spr->x_offs, ZOOM_LVL_GUI),
-					this->cursor_pos.y + this->cursor_sprite_pos[i].y + UnScaleByZoom(spr->y_offs, ZOOM_LVL_GUI),
-					ZOOM_LVL_GUI);
+			this->RenderOglSprite(spr, cs.image.pal,
+					this->cursor_pos.x + cs.pos.x + UnScaleByZoom(spr->x_offs, _gui_zoom),
+					this->cursor_pos.y + cs.pos.y + UnScaleByZoom(spr->y_offs, _gui_zoom),
+					_gui_zoom);
 		}
 	}
 }
 
+class OpenGLSpriteAllocator : public SpriteAllocator {
+public:
+	OpenGLSpriteLRUCache &lru;
+	SpriteID sprite;
+
+	OpenGLSpriteAllocator(OpenGLSpriteLRUCache &lru, SpriteID sprite) : lru(lru), sprite(sprite) {}
+protected:
+	void *AllocatePtr(size_t) override { NOT_REACHED(); }
+};
+
 void OpenGLBackend::PopulateCursorCache()
 {
-	static_assert(lengthof(_cursor.sprite_seq) == lengthof(this->cursor_sprite_seq));
-	static_assert(lengthof(_cursor.sprite_pos) == lengthof(this->cursor_sprite_pos));
-
 	if (this->clear_cursor_cache) {
 		/* We have a pending cursor cache clear to do first. */
 		this->clear_cursor_cache = false;
@@ -1103,21 +1108,15 @@ void OpenGLBackend::PopulateCursorCache()
 	}
 
 	this->cursor_pos = _cursor.pos;
-	this->cursor_sprite_count = _cursor.sprite_count;
 	this->cursor_in_window = _cursor.in_window;
 
-	for (uint i = 0; i < _cursor.sprite_count; ++i) {
-		this->cursor_sprite_seq[i] = _cursor.sprite_seq[i];
-		this->cursor_sprite_pos[i] = _cursor.sprite_pos[i];
-		SpriteID sprite = _cursor.sprite_seq[i].sprite;
+	this->cursor_sprites.clear();
+	for (const auto &sc : _cursor.sprites) {
+		this->cursor_sprites.emplace_back(sc);
 
-		if (!this->cursor_cache.Contains(sprite)) {
-			Sprite *old = this->cursor_cache.Insert(sprite, (Sprite *)GetRawSprite(sprite, SpriteType::Normal, &SimpleSpriteAlloc, this));
-			if (old != nullptr) {
-				OpenGLSprite *gl_sprite = (OpenGLSprite *)old->data;
-				gl_sprite->~OpenGLSprite();
-				free(old);
-			}
+		if (!this->cursor_cache.Contains(sc.image.sprite)) {
+			OpenGLSpriteAllocator allocator(this->cursor_cache, sc.image.sprite);
+			GetRawSprite(sc.image.sprite, SpriteType::Normal, &allocator, this);
 		}
 	}
 }
@@ -1127,12 +1126,7 @@ void OpenGLBackend::PopulateCursorCache()
  */
 void OpenGLBackend::InternalClearCursorCache()
 {
-	Sprite *sp;
-	while ((sp = this->cursor_cache.Pop()) != nullptr) {
-		OpenGLSprite *sprite = (OpenGLSprite *)sp->data;
-		sprite->~OpenGLSprite();
-		free(sp);
-	}
+	this->cursor_cache.Clear();
 }
 
 /**
@@ -1163,7 +1157,7 @@ void *OpenGLBackend::GetVideoBuffer()
 		this->vid_buffer = _glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_READ_WRITE);
 	} else if (this->vid_buffer == nullptr) {
 		_glBindBuffer(GL_PIXEL_UNPACK_BUFFER, this->vid_pbo);
-		this->vid_buffer = _glMapBufferRange(GL_PIXEL_UNPACK_BUFFER, 0, _screen.pitch * _screen.height * BlitterFactory::GetCurrentBlitter()->GetScreenDepth() / 8, GL_MAP_READ_BIT | GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_COHERENT_BIT);
+		this->vid_buffer = _glMapBufferRange(GL_PIXEL_UNPACK_BUFFER, 0, static_cast<GLsizeiptr>(_screen.pitch) * _screen.height * BlitterFactory::GetCurrentBlitter()->GetScreenDepth() / 8, GL_MAP_READ_BIT | GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_COHERENT_BIT);
 	}
 
 	return this->vid_buffer;
@@ -1264,25 +1258,13 @@ void OpenGLBackend::ReleaseAnimBuffer(const Rect &update_rect)
 	}
 }
 
-/* virtual */ Sprite *OpenGLBackend::Encode(const SpriteLoader::SpriteCollection &sprite, AllocatorProc *allocator)
+/* virtual */ Sprite *OpenGLBackend::Encode(SpriteType sprite_type, const SpriteLoader::SpriteCollection &sprite, SpriteAllocator &allocator)
 {
-	/* Allocate and construct sprite data. */
-	Sprite *dest_sprite = (Sprite *)allocator(sizeof(*dest_sprite) + sizeof(OpenGLSprite));
+	/* This encoding is only called for mouse cursors. We don't need real sprites but OpenGLSprites to show as cursor. These need to be put in the LRU cache. */
+	OpenGLSpriteAllocator &gl_allocator = static_cast<OpenGLSpriteAllocator&>(allocator);
+	gl_allocator.lru.Insert(gl_allocator.sprite, std::make_unique<OpenGLSprite>(sprite_type, sprite));
 
-	OpenGLSprite *gl_sprite = (OpenGLSprite *)dest_sprite->data;
-	new (gl_sprite) OpenGLSprite(sprite[ZOOM_LVL_NORMAL].width, sprite[ZOOM_LVL_NORMAL].height, sprite[ZOOM_LVL_NORMAL].type == SpriteType::Font ? 1 : ZOOM_LVL_END, sprite[ZOOM_LVL_NORMAL].colours);
-
-	/* Upload texture data. */
-	for (int i = 0; i < (sprite[ZOOM_LVL_NORMAL].type == SpriteType::Font ? 1 : ZOOM_LVL_END); i++) {
-		gl_sprite->Update(sprite[i].width, sprite[i].height, i, sprite[i].data);
-	}
-
-	dest_sprite->height = sprite[ZOOM_LVL_NORMAL].height;
-	dest_sprite->width  = sprite[ZOOM_LVL_NORMAL].width;
-	dest_sprite->x_offs = sprite[ZOOM_LVL_NORMAL].x_offs;
-	dest_sprite->y_offs = sprite[ZOOM_LVL_NORMAL].y_offs;
-
-	return dest_sprite;
+	return nullptr;
 }
 
 /**
@@ -1292,7 +1274,7 @@ void OpenGLBackend::ReleaseAnimBuffer(const Rect &update_rect)
  * @param y Y position of the sprite.
  * @param zoom Zoom level to use.
  */
-void OpenGLBackend::RenderOglSprite(OpenGLSprite *gl_sprite, PaletteID pal, int x, int y, ZoomLevel zoom)
+void OpenGLBackend::RenderOglSprite(const OpenGLSprite *gl_sprite, PaletteID pal, int x, int y, ZoomLevel zoom)
 {
 	/* Set textures. */
 	bool rgb = gl_sprite->BindTextures();
@@ -1333,7 +1315,7 @@ void OpenGLBackend::RenderOglSprite(OpenGLSprite *gl_sprite, PaletteID pal, int
 }
 
 
-/* static */ GLuint OpenGLSprite::dummy_tex[] = { 0, 0 };
+/* static */ std::array<GLuint, OpenGLSprite::NUM_TEX> OpenGLSprite::dummy_tex{};
 /* static */ GLuint OpenGLSprite::pal_identity = 0;
 /* static */ GLuint OpenGLSprite::pal_tex = 0;
 /* static */ GLuint OpenGLSprite::pal_pbo = 0;
@@ -1344,7 +1326,7 @@ void OpenGLBackend::RenderOglSprite(OpenGLSprite *gl_sprite, PaletteID pal, int
  */
 /* static */ bool OpenGLSprite::Create()
 {
-	_glGenTextures(NUM_TEX, OpenGLSprite::dummy_tex);
+	_glGenTextures(NUM_TEX, OpenGLSprite::dummy_tex.data());
 
 	for (int t = TEX_RGBA; t < NUM_TEX; t++) {
 		_glBindTexture(GL_TEXTURE_2D, OpenGLSprite::dummy_tex[t]);
@@ -1405,7 +1387,7 @@ void OpenGLBackend::RenderOglSprite(OpenGLSprite *gl_sprite, PaletteID pal, int
 /** Free all common resources for sprite rendering. */
 /* static */ void OpenGLSprite::Destroy()
 {
-	_glDeleteTextures(NUM_TEX, OpenGLSprite::dummy_tex);
+	_glDeleteTextures(NUM_TEX, OpenGLSprite::dummy_tex.data());
 	_glDeleteTextures(1, &OpenGLSprite::pal_identity);
 	_glDeleteTextures(1, &OpenGLSprite::pal_tex);
 	if (_glDeleteBuffers != nullptr) _glDeleteBuffers(1, &OpenGLSprite::pal_pbo);
@@ -1413,27 +1395,28 @@ void OpenGLBackend::RenderOglSprite(OpenGLSprite *gl_sprite, PaletteID pal, int
 
 /**
  * Create an OpenGL sprite with a palette remap part.
- * @param width Width of the top-level texture.
- * @param height Height of the top-level texture.
- * @param levels Number of mip-map levels.
- * @param components Indicates which sprite components are used.
+ * @param sprite The sprite to create the OpenGL sprite for
  */
-OpenGLSprite::OpenGLSprite(uint width, uint height, uint levels, SpriteColourComponent components)
+OpenGLSprite::OpenGLSprite(SpriteType sprite_type, const SpriteLoader::SpriteCollection &sprite)
 {
+	const auto &root_sprite = sprite.Root();
+	this->dim.width = root_sprite.width;
+	this->dim.height = root_sprite.height;
+	this->x_offs = root_sprite.x_offs;
+	this->y_offs = root_sprite.y_offs;
+
+	int levels = sprite_type == SpriteType::Font ? 1 : to_underlying(ZoomLevel::End);
 	assert(levels > 0);
 	(void)_glGetError();
 
-	this->dim.width = width;
-	this->dim.height = height;
-
-	MemSetT(this->tex, 0, NUM_TEX);
+	this->tex = {};
 	_glActiveTexture(GL_TEXTURE0);
 	_glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
 
 	for (int t = TEX_RGBA; t < NUM_TEX; t++) {
 		/* Sprite component present? */
-		if (t == TEX_RGBA && components == SCC_PAL) continue;
-		if (t == TEX_REMAP && (components & SCC_PAL) != SCC_PAL) continue;
+		if (t == TEX_RGBA && root_sprite.colours == SpriteComponent::Palette) continue;
+		if (t == TEX_REMAP && !root_sprite.colours.Test(SpriteComponent::Palette)) continue;
 
 		/* Allocate texture. */
 		_glGenTextures(1, &this->tex[t]);
@@ -1446,7 +1429,7 @@ OpenGLSprite::OpenGLSprite(uint width, uint height, uint levels, SpriteColourCom
 		_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
 		/* Set size. */
-		for (uint i = 0, w = width, h = height; i < levels; i++, w /= 2, h /= 2) {
+		for (int i = 0, w = this->dim.width, h = this->dim.height; i < levels; i++, w /= 2, h /= 2) {
 			assert(w * h != 0);
 			if (t == TEX_REMAP) {
 				_glTexImage2D(GL_TEXTURE_2D, i, GL_R8, w, h, 0, GL_RED, GL_UNSIGNED_BYTE, nullptr);
@@ -1456,12 +1439,18 @@ OpenGLSprite::OpenGLSprite(uint width, uint height, uint levels, SpriteColourCom
 		}
 	}
 
+	/* Upload texture data. */
+	for (ZoomLevel zoom = ZoomLevel::Min; zoom <= (sprite_type == SpriteType::Font ? ZoomLevel::Min : ZoomLevel::Max); ++zoom) {
+		const auto &src_sprite = sprite[zoom];
+		this->Update(src_sprite.width, src_sprite.height, to_underlying(zoom), src_sprite.data);
+	}
+
 	assert(_glGetError() == GL_NO_ERROR);
 }
 
 OpenGLSprite::~OpenGLSprite()
 {
-	_glDeleteTextures(NUM_TEX, this->tex);
+	_glDeleteTextures(NUM_TEX, this->tex.data());
 }
 
 /**
@@ -1529,7 +1518,7 @@ inline Dimension OpenGLSprite::GetSize(ZoomLevel level) const
  * Bind textures for rendering this sprite.
  * @return True if the sprite has RGBA data.
  */
-bool OpenGLSprite::BindTextures()
+bool OpenGLSprite::BindTextures() const
 {
 	_glActiveTexture(GL_TEXTURE0);
 	_glBindTexture(GL_TEXTURE_2D, this->tex[TEX_RGBA] != 0 ? this->tex[TEX_RGBA] : OpenGLSprite::dummy_tex[TEX_RGBA]);
