@@ -2,7 +2,7 @@
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
  * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <https://www.gnu.org/licenses/old-licenses/gpl-2.0>.
  */
 
 /** @file sdl2_v.h Base of the SDL2 video driver. */
@@ -17,9 +17,9 @@
 /** The SDL video driver. */
 class VideoDriver_SDL_Base : public VideoDriver {
 public:
-	VideoDriver_SDL_Base(bool uses_hardware_acceleration = false) : VideoDriver(uses_hardware_acceleration), sdl_window(nullptr), buffer_locked(false) {}
+	VideoDriver_SDL_Base(bool uses_hardware_acceleration = false) : VideoDriver(uses_hardware_acceleration) {}
 
-	const char *Start(const StringList &param) override;
+	std::optional<std::string_view> Start(const StringList &param) override;
 
 	void Stop() override;
 
@@ -41,14 +41,14 @@ public:
 
 	std::vector<int> GetListOfMonitorRefreshRates() override;
 
-	const char *GetInfoString() const override { return this->driver_info.c_str(); }
+	std::string_view GetInfoString() const override { return this->driver_info; }
 
 protected:
-	struct SDL_Window *sdl_window; ///< Main SDL window.
-	Palette local_palette; ///< Current palette to use for drawing.
-	bool buffer_locked; ///< Video buffer was locked by the main thread.
-	Rect dirty_rect; ///< Rectangle encompassing the dirty area of the video buffer.
-	std::string driver_info; ///< Information string about selected driver.
+	struct SDL_Window *sdl_window = nullptr; ///< Main SDL window.
+	Palette local_palette{}; ///< Current palette to use for drawing.
+	bool buffer_locked = false; ///< Video buffer was locked by the main thread.
+	Rect dirty_rect{}; ///< Rectangle encompassing the dirty area of the video buffer.
+	std::string driver_info{}; ///< Information string about selected driver.
 
 	Dimension GetScreenSize() const override;
 	void InputLoop() override;
@@ -73,7 +73,7 @@ private:
 	void LoopOnce();
 	void MainLoopCleanup();
 	bool CreateMainSurface(uint w, uint h, bool resize);
-	const char *Initialize();
+	std::optional<std::string_view> Initialize();
 
 #ifdef __EMSCRIPTEN__
 	/* Convert a constant pointer back to a non-constant pointer to a member function. */
@@ -83,9 +83,9 @@ private:
 	/**
 	 * This is true to indicate that keyboard input is in text input mode, and SDL_TEXTINPUT events are enabled.
 	 */
-	bool edit_box_focused;
+	bool edit_box_focused = false;
 
-	int startup_display;
+	int startup_display = 0;
 };
 
 #endif /* VIDEO_SDL_H */
